# Brasil-SR #
Brasil-SR is a spectral radiative transfer model that uses two-stream radiative transfer scheme to solve the radiative transfer equation for clear sky and overcast conditions.

## Dependencies:
* Brasil-SR requires the [NetCDF Fortran library](https://github.com/Unidata/netcdf-fortran).
* The model is optimized for the [Intel Fortran compiler](https://software.intel.com/en-us/fortran-compilers). All dependencies should be compiled using the Intel Parallel Studio compilers (*icc*, *icpc*, *ifort*).
* [Doxygen](http://www.doxygen.nl/) is required to generate the HTML documentation.

## Contribution guidelines:
Detailed contribution guidelines will be disclosed in the future, but here are a few conventions and guidelines.

### Coding conventions:
Brasil-SR is written in Fortran. While much of the code is older, the standard going forward should be Fortran-90. Try to optimize the code for readability.

* Use the Fortran 90 free format syntax.
* Indent blocks using two spaces (soft tabs).
* Use lower case for standard Fortran statements to improve readability.
* Use blank lines and spaces where appropriate to format your code.
* Wrap lines at 80 characters or less. Use the `&` operator if needed.
* Line up statements where appropriate.

### Documentation conventions:
Brasil-SR is a scientific software that will be used and improved by other researchers in the future. They should be able to understand your code and trace back the references you used to implement it.

* **Document your code** using [Doxygen](http://www.doxygen.nl/).
* Add BibTeX citations/references to `doxygen/references.bib`.
* If a routine or data set is implemented based on a specific paper, then its documentation should mention the source using the `@cite` keyword.

### Repository conventions:

#### Atomic commits:
The idea behind *atomic* changes is that each commit should revolve around a single task. You should commit each task separately in order to make the project more flexible for bug fixes, branch merges, and rollbacks.
Note that a single task *can* consist of changes in multiple files.

- Commit each fix or task as a separate change.
- Only commit when a block of work is complete.

#### Git commit messages:
- Limit the subject line to 50 characters.
- Capitalize the subject line (*Add feature*, not *add feature*).
- Use the present tense in the subject line (*Add feature*, not *Added feature*).
- Use the imperative mood in the subject line (*Move cursor to...*, not *Moves cursor to...*).
- Do not end the subject line with a period.
- Separate subject from body with a blank line.
- Use the body to explain what and why vs. how.
- If your commit solves an open issue, reference its number at the end (i.e.: `Resolves: #1`).

Here's an example (adapted from [this post](https://chris.beams.io/posts/git-commit/)):

```
Summarize changes in around 50 characters or less

More detailed explanatory text, if necessary. In some contexts, the
first line is treated as the subject of the commit and the rest of
the text as the body. The blank line separating the summary from the
body is critical (unless you omit the body entirely); various tools
like 'log', 'shortlog' and 'rebase' can get confused if you run the
two together.

Explain the problem that this commit is solving. Focus on why you
are making this change as opposed to how (the code explains that).
Are there side effects or other unintuitive consequences of this
change? Here's the place to explain them.

If your commit is related to an open issue on the issue tracker
(and it should!), put references to them at the bottom, like this:

Resolves: #123
See also: #456, #789
```

## Contact:
* Brasil-SR is maintained by the [LABREN](http://labren.ccst.inpe.br) team at [CCST](http://www.ccst.inpe.br/)/[INPE](http://www.inpe.br/).
* If you have any issues with this repository, please report them to [mauricio.zarzur@inpe.br](mailto:mauricio.zarzur@inpe.br).
