!-------------------------------------------------------------------------------
! LABREN: Laboratory of Modeling and Studies on Renewable Energy Resources
! http://labren.ccst.inpe.br/index-en.html
!-------------------------------------------------------------------------------
!Module that handles BRASIL-SR options common between modules
!!@details Domain configuration, grid points with local data, grid points to save spectral output
!! and model configuration options can be defined at this module
!!
!! A full description of the model and benchmark statistics can be found at:
!!
!! Assessment of the impact of biomass burning aerosols in spectral and broadband
!! downward surface solar irradiance - a case study in Brazilian Amazon and Cerrado.
!! Madeleine S.G. Casagrande, Fernando R. Martins, Nilton M.E. Rosário, Francisco J.L. de Lima,
!! André R. Gonçalves, Rodrigo S. Costa, Antonio Maurício Zarzur, Marcelo P. Pes and Enio B. Pereira.
!! Submitted to Remote Sensing on Semptember 25th, 2021.
!!
!! @author     LABREN
!! @version    2.0
!! @date       2021
!-------------------------------------------------------------------------------
MODULE gradevars
IMPLICIT NONE

!GOES13 and other input files grid size, do not modify
   INTEGER, PARAMETER :: NLONGI = 1784
   INTEGER, PARAMETER :: NLATI = 1180
!-----------------------------------------------------
!From here on, configuration options

!! DNI obtained without delta-Eddington scaling is more accurate.
!! However, delta-scaled DNI (and corresponding diffuse irradiation) can be extracted with OUTPUT_SCALED_DNI=.TRUE.
   LOGICAL, PARAMETER :: OUTPUT_SCALED_DNI=.TRUE.
!! To run in clear sky mode, set  RUN_CS_ONLY=.TRUE., otherwise CEFF information is required
   LOGICAL, PARAMETER :: RUN_CS_ONLY=.TRUE.
!! IF  RUN_ALL_HOURS=.FALSE., the model will be executed for hours specified in imagens<month><year>.dat
   LOGICAL, PARAMETER :: RUN_ALL_HOURS=.TRUE.
! FREQUENCY in minutes between desired time steps (clear sky) or minimum time interval between satellite images
!will determine the size of time arrays (Ntimes)
!! Ntimes = (60/ FREQUENCY)*14+1
!1 min
   ! character (len = *), PARAMETER :: FREQUENCY = '1'
   ! INTEGER, PARAMETER::IntFREQUENCY=1
   ! INTEGER, PARAMETER :: Ntimes = 841

!5mins
   ! character (len = *), PARAMETER :: FREQUENCY = '5'
   ! INTEGER, PARAMETER::IntFREQUENCY=5
   ! INTEGER, PARAMETER :: Ntimes = 169
!15 mins
   ! character (len = *), PARAMETER :: FREQUENCY = '15'
   ! INTEGER, PARAMETER::IntFREQUENCY=15
      ! INTEGER, PARAMETER :: Ntimes = 57
! !30 mins
   character (len = *), PARAMETER :: FREQUENCY = '30'
   INTEGER, PARAMETER::IntFREQUENCY=30
   INTEGER, PARAMETER :: Ntimes = 29

  ! Flag USEDIARIO, to be used when  RUN_CS_ONLY=.FALSE.
   !Daily(True)/Mean monthly(False) transmittances
  LOGICAL, PARAMETER :: USEDIARIO = .TRUE.
  ! Flag USEMEANAOD, to be used when  USEDIARIO=.TRUE.
   !Daily(True)/Mean monthly(False) AOD values
  LOGICAL, PARAMETER :: USEMEANAOD = .FALSE.
  !If USEMEANAOD = .TRUE. uncomment below
  ! INTEGER, PARAMETER :: Ntimes = 1

! Flag USE_KAEXT_MAXSECOND: Assume maximum Kext in second layer (True) or first layer (False) from surface, in AOD vertical distribution
   LOGICAL, PARAMETER :: USE_KAEXT_MAXSECOND = .TRUE.

! Flag USE_SPECTRAL_ALB: Use spectral (True) or broadband (False) albedo
   LOGICAL, PARAMETER :: USE_SPECTRAL_ALB = .TRUE.

! Flag RUN_REGIONAL: Run for domain (True) or not
   LOGICAL, PARAMETER :: RUN_REGIONAL = .TRUE.

   character (len = 5), parameter ::  CN_LABEL  = "_brdf"
   character (len = 25), parameter :: TRANS_LABEL  = '_insitu'
   ! character (len = 25), parameter :: TRANS_LABEL  = '_regional'


   !! NOTE2:num_points is the number of grid points for which input is entered, while run_points is the number of grid points for
     ! which transmittance and spectral output is calculated and saved. They might be coincident or not.
! Flag USEAERONETIJ: Use AERONET for selected grid points (the number of grid points with aeronet input is given by num_points)
  LOGICAL, PARAMETER :: USEAERONETIJ = .TRUE.
  INTEGER, PARAMETER :: num_points=3
! Flag SAVEIJ: Save grid point output, including spectral output (the number of grid points output saved is given by run_points)
  LOGICAL, PARAMETER :: SAVEIJ = .TRUE.
  INTEGER, PARAMETER :: run_points=4


!! NOTE1: If  RUN_REGIONAL = .FALSE. and SAVEIJ = .FALSE., the transmittance module will never run and no output will be generated

  !Cell size in lat lon (GOES13)
  DOUBLE PRECISION, PARAMETER :: cellsizelat=0.05, cellsizelon=0.03

!LMAX, L0 and  associated parameters for irradiance calculation (not used when running in clear sky mode)
   LOGICAL, PARAMETER :: USELMAX = .FALSE. ! Use statistic LMAX 1, or use max rad 0
   LOGICAL, PARAMETER :: USEL0 = .FALSE.  ! Use statistic LMAX 1, or use max rad 0
   INTEGER, PARAMETER :: LMAX = 7000 ! Lmax TS
   ! REAL, PARAMETER :: TAUWLMAX = 100.0D0 ! Lmax TS
   !Uncomment when using USEL0 = 0
   INTEGER :: L0CEFF
   DOUBLE PRECISION :: TAUWL0
   !Uncomment the corresponding two lines when using USEL0 = 1
   ! INTEGER, PARAMETER :: L0CEFF = 2000 ! Vis L0
   ! DOUBLE PRECISION, PARAMETER :: TAUWL0 = 4.5D0 ! Lmax TS

   ! INTEGER, PARAMETER :: L0CEFF = 2700 ! Vis L0
   ! DOUBLE PRECISION, PARAMETER :: TAUWL0 = 7.0D0 ! Lmax TS
   !
   ! INTEGER, PARAMETER :: L0CEFF = 3400 ! Vis L0
   ! DOUBLE PRECISION, PARAMETER :: TAUWL0 = 15.0D0 ! Lmax TS

!Uncomment correspondig section to run a subdomain
!Biomass burning region subdomain
INTEGER, PARAMETER :: IMIN =         701
INTEGER, PARAMETER :: JMIN =         301
INTEGER, PARAMETER :: IRANGE =         533
INTEGER, PARAMETER :: JRANGE =         299
INTEGER, PARAMETER :: IMAX =        1233 ! VERIFICAR, IMAX = IMIN + IRANGE -1
INTEGER, PARAMETER :: JMAX =         599  ! VERIFICAR, JMAX = JMIN + JRANGE -1


! Atlas subdomain
!    INTEGER, PARAMETER :: IMIN = 301
!    INTEGER, PARAMETER :: JMIN = 154
!    INTEGER, PARAMETER :: IRANGE = 1317
!    INTEGER, PARAMETER :: JRANGE = 785
!    INTEGER, PARAMETER :: IMAX =  1617 ! VERIFICAR, IMAX = IMIN + IRANGE -1
!    INTEGER, PARAMETER :: JMAX =  938  ! VERIFICAR, JMAX = JMIN + JRANGE -1

!full grid
  ! INTEGER, PARAMETER :: IMIN = 1
  ! INTEGER, PARAMETER :: JMIN = 1
  ! INTEGER, PARAMETER :: IRANGE = 1784
  ! INTEGER, PARAMETER :: JRANGE = 1180
  ! INTEGER, PARAMETER :: IMAX =  1784 ! VERIFICAR, IMAX = IMIN + IRANGE -1
  ! INTEGER, PARAMETER :: JMAX =  1180  ! VERIFICAR, JMAX = JMIN + JRANGE -1

END MODULE
