BRASIL-SR documentation
=================================

The BRASIL-SR clear sky model, its required input data and the skill of the model in clear sky conditions are describe in the paper:

Assessment of the impact of biomass burning aerosols in spectral and broadband downward surface solar irradiance - a case study in Brazilian Amazon and Cerrado. Madeleine S.G. Casagrande, Fernando R. Martins, Nilton M.E. Rosário, Francisco J.L. de Lima, André R. Gonçalves, Rodrigo S. Costa, Antonio Maurício Zarzur, Marcelo P. Pes and Enio B. Pereira. Submitted to Remote Sensing.

Other useful references are @cite atlas2017, @cite martins2006parameterization, @cite martins2008solar, @cite costa2016

This document describe more technical details that could be useful to the user as a reference.

Model dimensions
----------------
### Solar spectrum #######

Brasil-SR model divides the solar spectrum (0.2 -3.5 \f$\mu m\f$) into 37 wavelength intervals, further divided into a total of 151 pseudos-intervals (new version) or 135 (old version). It is possible to run for a range of the spectrum, selecting a beginning and ending wavelength values among the 37, or for the whole spectrum (1-37). For example, to run for the PAR wavelength interval, (16-22) must be selected.

The beginning (IBEG) and ending values (ISTO) are selected among vectors INTVA and INTVE (in STRPSRB subroutine) using INTVAL (set on subroutine D2STR).

*    MAXWEL=37: wavelength interval maximum dimension
*    MAXIND= 135: wavelength sub-interval maximum dimension

The INDEX variable, constructed from IH, defines to which of the 37 intervals is assigned each of the 135 sub-intervals. In particular, the first 22 intervals consist of only one sub-interval.

Typically, the following variables are used as counters for intervals and sub-intervals:

*    IALL    : counter of spectral sub-interval
*    K, KA    : typically, refer to spectral interval that is being calculated


<table> <caption id="lambdaint">Spectral intervals in Brasil-SR</caption> <tr><th>Interval <th> Central \f$ \lambda \f$ <th> Spectral solar constant <tr><td> 1 <td> 0.200 <td> 0.4 <tr><td> 2 <td> 0.210 <td> 16.0 <tr><td> 3 <td> 0.220 <td> 37.0 <tr><td> 4 <td> 0.230 <td> 55.0 <tr><td> 5 <td> 0.240 <td> 54.0 <tr><td> 6 <td> 0.250 <td> 60.0 <tr><td> 7 <td> 0.260 <td> 86.0 <tr><td> 8 <td> 0.270 <td> 160.0 <tr><td> 9 <td> 0.280 <td> 177.0 <tr><td> 10 <td> 0.290 <td> 273.0 <tr><td> 11 <td> 0.300 <td> 757.5 <tr><td> 12 <td> 0.320 <td> 1476.0 <tr><td> 13 <td> 0.340 <td> 1845.4 <tr><td> 14 <td> 0.360 <td> 2107.7 <tr><td> 15 <td> 0.380 <td> 3246.8 <tr><td> 16 <td> 0.400 <td> 4280.7 <tr><td> 17 <td> 0.450 <td> 9476.2 <tr><td> 18 <td> 0.500 <td> 9601.2 <tr><td> 19 <td> 0.550 <td> 9320.5 <tr><td> 20 <td> 0.600 <td> 8836.5 <tr><td> 21 <td> 0.650 <td> 7960.5 <tr><td> 22 <td> 0.700 <td> 3667.6 <tr><td> 23 <td> 0.735 <td> 7949.8 <tr><td> 24 <td> 0.765 <td> 1220.4 <tr><td> 25 <td> 0.778 <td> 1791.7 <tr><td> 26 <td> 0.825 <td> 8769.6 <tr><td> 27 <td> 0.877 <td> 1958.5 <tr><td> 28 <td> 0.950 <td> 10184.4 <tr><td> 29 <td> 1.040 <td> 5094.8 <tr><td> 30 <td> 1.150 <td> 7307.3 <tr><td> 31 <td> 1.245 <td> 2996.9 <tr><td> 32 <td> 1.430 <td> 10003.8 <tr><td> 33 <td> 1.625 <td> 2591.6 <tr><td> 34 <td> 1.890 <td> 6498.2 <tr><td> 35 <td> 2.150 <td> 796.6 <tr><td> 36 <td> 2.590 <td> 3813.9 <tr><td> 37 <td> 3.280 <td> 1128.3 </table>


### Atmosphere #######

The atmosphere in Brasil-SR is divided in 50 vertical layers between 0 and 50 km of altitude (values from Table 1 \cite mcclatchey1972optical, pressure), allowing for a maximum of 50 layers given by LAYMAX (51 pressure levels at boundaries, LEVMAX). The number of these layers that is in use is given by NLAY, and defined through 31 pressure levels (NLEV) (TPRESS variable in LESEN subroutine, STRPSRB.f90 file). Pressure, temperature, water vapor and ozone profiles follow @cite Anderson1986 but are corrected with input data.

###Cloud droplet distributions #######

Brasil-SR allows for the selection of 8 types of cloud droplet distributions, after Table 1. @cite stephens1978a :

    Stratus II (ST II)
    Stratocumulus I (SC I)
    Altostratus (As)
    Stratocumulus II (SC II)
    Fair weather cumulus
    Cumulusnimbus (Cb)
    Stratus I (ST I)
    Nimbostratus (Nb)

The extinction coefficient (EWOL), asymmetry factor (GWOL) and AWOL (1-\f$\omega\f$) are defined for each cloud droplet distributions and 16 solar radiation wavelength intervals. The variable INWOSW acts as an index that defines to which of these 16 wavelength intervals is allocated each of the 151 (135 in the old version) pseudo-intervals of the model. (Values were compared to those reported in @cite stephens1979optical , with consistency).

The cloud distribution is selected with variable IWP, and fixed to value IWP=3 (Altostratus - As) in D2STR.

###Aerosol models #######

Currently, Brasil-SR include 5 aerosol models (1: stratospheric; 2: marine; 3: continental polluted; 4: Amazon; 5: Cerrado), each one with a set of single scattering albedo (OMAER) and asymmetry parameter (GAER) values defined for each of the 37 wavelength intervals. The selection of aerosol types 2 to 5 is made based on the cell biome, while aerosol 1 is selected for heights>10 km. The weight variable for the AOD EXAER, =1 for 550, is modified using the input aerosol Angstrom exponent.

###Aerosol optical depth assimilation for type 1 aerosol profile###

Brasil-SR uses the extinction coefficient \f$K_{ext} [km^{-1}]\f$ in each layer to account for the aerosol impact.

The model includes a vertical profile for \f$K_{ext}\f$ between 0 and 51 (SCAL), where the first position of the array correspond to the height 50 km and the last to 0 km. The first 5th values (from 0 to 5 km) are modified to account for the local aerosol load. Originally, this was made estimating \f$K_{ext}(0)\f$ from the meteorological range or horizontal visibility, as in: \f[ K_{ext}(0)=3.912/VIS \f] where is considered that the width of the first layer is 1km. After the estimation of \f$K_{ext}(0)\f$, an interpolation between \f$K_{ext}(0)\f$ and \f$K_{ext}(5)\f$ was made following an inverse exponential rule: \f[ K_{ext}(i)=K_{ext}(0)e^{Cscal*h} \f] where h is the corresponding height of the layer (equal to \f$i\f$ since the width of all layers is 1km) and \f$Cscal=\frac{1}{H}\ln{\frac{K_{ext}(5)}{K_{ext}(0)}}\f$

After issue2, \f$K_{ext}(0)\f$ is instead estimated from the aerosol optical depth \f$\tau\f$, following the procedure described below.

According to its definition, \f$\tau=\int_{0}^{\infty} K_{ext}dz \f$, that in this case reduces to \f$\tau=\sum_{i=0}^{51} K_{ext}(i)\f$ (since \f$\Delta z_{i} = 1 \f$ for all model layers). \f$K_{ext}\f$ is corrected only for the first 5 km, so the contribution of the last 45 levels should be removed from \f$\tau\f$, and \f$\tau_{0-5km}=\sum_{i=0}^{4} K_{ext}(i)=\sum_{i=0}^{4} K_{ext}(0)e^{Cscal*h}\f$.

Using \f$\sum_{i=0}^{n-1}ar^{k}=a\frac{1-r^{n}}{1-r}\f$, follows that:

\f[ \tau_{0-5km}=\sum_{i=0}^{4} K_{ext}(0) e^{Cscal*i} = K_{ext}(0)\frac{1-{e^{Cscal}}^5}{1-e^{Cscal}} \f] so \f[ K_{ext}(0)= \tau_{0-5km}\frac{1-e^{Cscal}}{1-{e^{Cscal}}^5} \f] where the implicit dependency of \f$Cscal\f$ with \f$K_{ext}(0)\f$ is first ignored.

Using this relation, \f$K_{ext}(0)\f$ is estimated with an iterative procedure, starting with an initial \f$K_{ext}(0)=0.6\tau_{0-5km}\f$, that is used to estimate \f$Cscal\f$ and the remaining \f$K_{ext}(i)\f$. After this, a new value for the AOD for the first 5 km is calculated as \f$\tau_{calc_0-5km}=\sum_{i=0}^{4}K_{ext}(i)\f$ and compared with the target value \f$\tau_{0-5km}\f$. This cycle repeats until \f$\tau_{0-5km}-\tau_{calc_0-5km}<=0.00001\f$.
