!-------------------------------------------------------------------------------
! LABREN: Laboratório de Modelagem e Estudos de Recursos Renováveis de Energia
!-------------------------------------------------------------------------------
! SUBROUTINE: SUNRISE
!
!> Calculates time for sunrise

!! @param[in] DECR  DOUBLE      Solar declination [radians].
!! @param[in] YLATR  DOUBLE     latitude of grid point (radians)
!! @param[out] TSRA   DOUBLE     apparent time for sunrise (hours)
!! @see        SWTR8A
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
 SUBROUTINE SUNRISE(DECR,YLATR,TSRA)
   use BRconstants
!     calculate time for sunrise

!  ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

!     DECR   : solar declination (radians)
!     YLATR  : latitude of grid point (radians)
!     CDR    : change degrees to radians
!     COWSR  : cossine of sunrise hour angle
!     WSR    : sunrise hour angle (degrees)
!     TSRA   : apparent time for sunrise (hours)
!......................................................................

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      PI    = DACOS(-1.0_dp)
      CDR  = PI/180.0_dp
!     calculate hour for sunrise and tests if is not a polar
!     winter or summer

      COWSR = -1.0_dp*DTAN(DECR)*DTAN(YLATR)
       IF (COWSR .LT. -1.0_dp) THEN
         WRITE(*,*) ' NO SUNSET   NO SUNRISE: 24 HOUR INSOLATION'
         TSRA  =  0.0_dp
       ELSE IF (COWSR .GT.  1.0_dp) THEN
         WRITE(*,*) ' DARK SIDE OF THE EARTH  NO INSOLATION'
         STOP
       ELSE
         WSR   = ACOS(COWSR)/CDR
         TSRA  = 12.00_dp - WSR/15._dp
       END IF
      RETURN
      END
