!-------------------------------------------------------------------------------
! LABREN: Laboratory of Modeling and Studies on Renewable Energy Resources
! http://labren.ccst.inpe.br/index-en.html
!-------------------------------------------------------------------------------
! PROGRAM: BRASILSR_irradiances
!> Calulates global, diffuse and direct irradiation for clear sky (RUN_CS_ONLY=.TRUE.)
!! or considering clouds, for a domain in a given day.
!!
!! Domain configuration, grid points with local data, grid points to save spectral output
!! and model configuration options can be found at the configvars module (see configvars directory in base dir)
!!
!! A full description of the model and benchmark statistics can be found at:
!!
!! Assessment of the impact of biomass burning aerosols in spectral and broadband
!! downward surface solar irradiance - a case study in Brazilian Amazon and Cerrado.
!! Madeleine S.G. Casagrande, Fernando R. Martins, Nilton M.E. Rosário, Francisco J.L. de Lima,
!! André R. Gonçalves, Rodrigo S. Costa, Antonio Maurício Zarzur, Marcelo P. Pes and Enio B. Pereira.
!! Submitted to Remote Sensing on Semptember 25th, 2021.
!!
!! @author     LABREN
!! @version    2.0
!! @date       2021
!-------------------------------------------------------------------------------
PROGRAM BRASILSR_irradiances
use netcdf
use omp_lib
USE IFPORT
use gradevars
use BRconstants
use BRinterface
!IMPLICIT DOUBLE PRECISION (A-H,O-Z)
CHARACTER*1 CJDAY
CHARACTER*2 CCJDAY
CHARACTER*3 CMON, CCCJDAY,CCHORI,JDAY, JDAYREAD
CHARACTER*4 CYEAR,CHORI,CL0CEFF,CLMAX
CHARACTER*6 CTAUWL0
CHARACTER*190 CDIR, CFILE,CDESC,CANAL,CSAT,CLATI,CLONG
CHARACTER*190 CKD,  CTRANS
CHARACTER*190 arq_nc,CIMAG,CKT,  CIMAGEM
CHARACTER*300 transdir,ktdir,kddir,solardir
character*80  fmax
DOUBLE PRECISION, PARAMETER:: offset_factor=0.D0, scale_factor=1.D0
INTEGER :: IYEAR, IMON, NDIA
logical :: fileExists, dirExists, dirExists2
LOGICAL(4) result
character (len = *), parameter :: UNITS     = "units"
character (len = *), parameter :: LONGNAME  = "long_name"
character (len = *), parameter :: SCALE     = "scale_factor"
character (len = *), parameter :: OFFSET     = "add_offset"

character (len = *), parameter :: LAT_NAME  = "latitude"
character (len = *), parameter :: LON_NAME  = "longitude"
character (len = *), parameter :: LAT_UNITS = "degrees_north"
character (len = *), parameter :: LON_UNITS = "degrees_east"
character (len = *), parameter :: GLO_NAME  = "GHI"
character (len = *), parameter :: KT_NAME   = "KT"
character (len = *), parameter :: KD_NAME   = "KD"
character (len = *), parameter :: DIF_NAME  = "DHI"
character (len = *), parameter :: DNI_NAME  = "DNI"
character (len = *), parameter :: DIF_SCALED_NAME  = "DHI_scaled"
character (len = *), parameter :: DIR_SCALED_NAME  = "DNI_scaled"
character (len = *), parameter :: GLO_LONG_NAME  = "Global horizontal irradiance"
character (len = *), parameter :: KT_LONG_NAME   = "Clearness index"
character (len = *), parameter :: KD_LONG_NAME   = "Diffuse fraction"
character (len = *), parameter :: DIF_LONG_NAME  = "Diffuse horizontal irradiance"
character (len = *), parameter :: DNI_LONG_NAME  = "Direct normal irradiance"
character (len = *), parameter :: DIF_SCALED_LONG_NAME  = "DHI obtained from scaled DNI"
character (len = *), parameter :: DIR_SCALED_LONG_NAME  = "Delta-Eddincton scaled DNI"
! character (len = *), parameter :: PAR_NAME  = "par"

character (len = *), parameter :: KT_UNITS  = ""
character (len = *), parameter :: KD_UNITS  = ""

character (len = *), parameter :: GLO_UNITS  = ""
character (len = *), parameter :: DIF_UNITS  = ""
character (len = *), parameter :: DIR_UNITS  = ""
! character (len = *), parameter :: PAR_UNITS  = ""
character (len = *), parameter :: VERSION_NAME  = "Brasil-SR_2.0"

CHARACTER*200:: VERSION_RESUME

double precision XXKT(IRANGE,JRANGE)
DOUBLE PRECISION GLOBAL(IRANGE,JRANGE), DIF(IRANGE,JRANGE), DIR(IRANGE,JRANGE), DIF_SCALED(IRANGE,JRANGE), DIR_SCALED(IRANGE,JRANGE), COSZEN(IRANGE,JRANGE),XHOR,LAT(JRANGE),LON(IRANGE)
DOUBLE PRECISION XLAT(IRANGE,JRANGE),XLON(IRANGE,JRANGE),XIMG(IRANGE,JRANGE),CEFF(IRANGE,JRANGE),PEFF(IRANGE,JRANGE)
DOUBLE PRECISION XTCR(IRANGE,JRANGE),XTCD(IRANGE,JRANGE),XTCDL0(IRANGE,JRANGE), XTDIR(IRANGE,JRANGE), XTDIRS(IRANGE,JRANGE)
DOUBLE PRECISION XKT(IRANGE,JRANGE), XKD(IRANGE,JRANGE)

integer :: glo_varid, dif_scaled_varid, dif_varid, dir_scaled_varid, dir_varid
INTEGER*2 LEITURACEFF(IRANGE,JRANGE), LEITURACEFFL0(IRANGE,JRANGE), LEITURAPEFF(IRANGE,JRANGE)
INTEGER*2 IKD(IRANGE,JRANGE), IKT(IRANGE,JRANGE)
INTEGER*2 IDIF_SCALED(IRANGE,JRANGE),IDIR_SCALED(IRANGE,JRANGE),IDIF(IRANGE,JRANGE),IDIR(IRANGE,JRANGE),IGLO(IRANGE,JRANGE)
INTEGER ERRO, IAUX, chunk

integer :: ndims_in, nvars_in, ngatts_in, unlimdimid_in,ncid,ncid1
integer, parameter :: NLAT = JRANGE, NLON = IRANGE
integer :: lat_dimid, lon_dimid
integer :: lat_varid, lon_varid, ceff_varid, ceffl0_varid, peff_varid
integer :: tdir_varid, tdirs_varid,  tclear_varid, tcloud_varid, tcloudL0_varid
integer ::scalelength
DOUBLE PRECISION , dimension(:), allocatable:: ceff_scale_factor, ceffl0_scale_factor, peff_scale_factor

INTEGER*2 IHOR(Ntimes)
LOGICAL RUN4IHOR(Ntimes)

integer(kind=4),dimension(2) :: dimids

integer*2 minimo(IRANGE,JRANGE)
integer :: min_varid
! character (len = 5), parameter ::  CN_LABEL  = "_brdf"
! character (len = 10), parameter :: TRANS_LABEL  = "_t030_brdf"


IF(USELMAX)THEN
  WRITE(CLMAX,'(I4)') LMAX
ELSE
  CLMAX='orig'
ENDIF

IF(USEL0)THEN
  WRITE(CTAUWL0,'(F6.3)') TAUWL0
  WRITE(CL0CEFF,'(I4)') L0CEFF
  VERSION_RESUME='Version release with L0 = '//CL0CEFF//', TAUWL0 = '//CTAUWL0//' and LMAX = '//CLMAX
ELSE
  CL0CEFF='noL0'
  VERSION_RESUME='Version release without L0 and with LMAX = '//CLMAX
ENDIF

print*,VERSION_RESUME

!.....input from description parameters
  OPEN(UNIT=10,FILE='gkss.dat')
  DO
      READ(10,1001,IOSTAT=IERR) CDIR
      IF (IERR /= 0) EXIT
      READ(10,*) CMON,IYEAR,JDAYREAD
      WRITE(CYEAR,'(I4)') IYEAR
  END DO
  CLOSE(10)


RUN4IHOR=.FALSE.

timeshour=(60/IntFREQUENCY)
DO i=8, 22
  DO j=1, timeshour
    pos=(i-8)*timeshour+j
    IHOR(pos)=i*100 + (j-1)*IntFREQUENCY
  END DO
END DO
JJJ=Ntimes+1

IF (RUN_ALL_HOURS) THEN
  RUN4IHOR=.TRUE. !Run for all timesteps
  JDAY=JDAYREAD
ELSE !Not running in clear sky mode, setting RUN4IHOR to .TRUE. when satellite image is available
  CIMAGEM='imagens'//CMON//CYEAR//'.dat'
  PRINT*, CIMAGEM
  !ABRINDO LISTA COM IMAGENS
  OPEN(UNIT=14,FILE=CIMAGEM)
  JJJ=1
  DO
     READ(14,*,IOSTAT=IERRO) JDAY,IYEAR,IHORI
     IF (IERRO /= 0) EXIT

     IF (USEDIARIO .AND. (JDAY.NE.JDAYREAD)) THEN
       print*, 'Previously running for day ',JDAYREAD,', now detecting day ', JDAY
       STOP 'ERROR: Trying to run with data for different days with option USEDIARIO = .TRUE.'
     END IF

     JJ=JJJ
     DO KK=JJ,Ntimes
       IF(int(IHORI).EQ.IHOR(KK)) THEN
         RUN4IHOR(KK)=.TRUE.
         JJJ=JJ+1
         JJ=JJ+1
         EXIT
        ELSEIF(int(IHORI).LT.IHOR(KK)) THEN
          JJ=JJ+1
          CONTINUE
        ELSEIF(int(IHORI).EQ.IHOR(KK)) THEN
           EXIT
        ENDIF
       ! print*,KK, JJ, IHOR(KK), int(IHORI),RUN4IHOR(KK)
     END DO
  END DO
  CLOSE (14)
endif


CALL MONTH(CMON,IMON)
READ(JDAYREAD,'(I3)')NDIA


    transdir=TRIM(CDIR)//CYEAR//'/'//CMON//'/trans'//CL0CEFF//'_'//CLMAX//TRANS_LABEL
    DO K=1,NTimes
      IF (RUN4IHOR(K)) THEN

          XHOR=DINT(IHOR(K)/100.0_dp)
          XHOR=XHOR+(IHOR(K)-100.0_dp*XHOR)/60.0_dp

    !    ABERTURA DOS ARQUIVOS DE TRANSMITANCIAS

          IF(IHOR(K).LT.1000) THEN
             WRITE(CCHORI,'(I3)') IHOR(K)
             CHORI='0'//CCHORI
          ELSE
             WRITE(CHORI,'(I4)') IHOR(K)
          ENDIF

          IF (USEDIARIO) THEN
            CTRANS=TRIM(transdir)//'/day'//JDAYREAD//'/trans'//CMON//CYEAR//JDAYREAD//'.'//CHORI//'.nc'
          ELSE
            CTRANS=TRIM(transdir)//'/trans'//CMON//CYEAR//'.'//CHORI//'.nc'
          ENDIF

          inquire( file=CTRANS, exist=fileExists )
          IF (fileExists) THEN
            WRITE(*,*)'Opening transmittance file ', trim(CTRANS)

            call check(nf90_open(CTRANS, nf90_write, ncid))
            call check(nf90_inquire(ncid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))
            call check(nf90_inq_varid(ncid, "latitude", lat_varid))
            call check(nf90_inq_varid(ncid, "longitude", lon_varid))
            call check(nf90_inq_varid(ncid, "TDIR",   tdir_varid))
            IF (OUTPUT_SCALED_DNI) THEN
              call check(nf90_inq_varid(ncid, "TDIR_scaled",   tdirs_varid))
            ENDIF
            call check(nf90_inq_varid(ncid, "TCLEAR", tclear_varid))
            IF(USEL0)THEN
              call check(nf90_inq_varid(ncid, "TCLOUD_at_L0", tcloudL0_varid))
            ENDIF
            call check(nf90_inq_varid(ncid, "TCLOUD", tcloud_varid))
            call check(nf90_inq_dimid(ncid, LAT_NAME, lat_dimid))
            call check(nf90_inq_dimid(ncid, LON_NAME, lon_dimid))
            call check(nf90_inquire_dimension(ncid,lat_dimid,len=ilat))
            call check(nf90_inquire_dimension(ncid,lon_dimid,len=ilon))
            call check(nf90_get_var(ncid,lat_varid,LAT))
            call check(nf90_get_var(ncid,lon_varid,LON))
            call check(nf90_get_var(ncid,tdir_varid,XTDIR))
            IF (OUTPUT_SCALED_DNI) THEN
              call check(nf90_get_var(ncid,tdirs_varid,XTDIRS))
            ENDIF
            call check(nf90_get_var(ncid,tclear_varid,XTCR))
            IF(USEL0)THEN
               call check(nf90_get_var(ncid,tcloudL0_varid,XTCDL0))
            ENDIF
            call check(nf90_get_var(ncid,tcloud_varid,XTCD))
            call check(nf90_close(ncid) )
          ELSE
            WRITE(*,*)'Transmittance file not found: ', trim(CTRANS)
          END IF


          ! Irradiance calculation


      IF (.NOT.RUN_CS_ONLY) THEN
        PRINT*,'Will attempt to look for cloud cover files in :', TRIM(CDIR)//CYEAR//'/'//CMON//'/images/cn'//CL0CEFF//'_'//CLMAX//CN_LABEL
        !alterar a leitura
        CFILE=TRIM(CDIR)//CYEAR//'/'//CMON//'/images/cn'//CL0CEFF//'_'//CLMAX//CN_LABEL//'/cn'//JDAYREAD// CYEAR//'.'//CHORI//'.nc'
        inquire( file=CTRANS, exist=fileExists )
        IF (fileExists) THEN
          WRITE(*,*)'Opening Effective cloud index file ', trim(CFILE)
          call check(nf90_open(CFILE, nf90_write, ncid))
          call check(nf90_inquire(ncid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))
          call check(nf90_inq_varid(ncid, "latitude", lat_varid))
          call check(nf90_inq_varid(ncid, "longitude", lon_varid))
          call check(nf90_inq_varid(ncid, "CEFF", ceff_varid))
          call check(nf90_inquire_attribute(ncid, ceff_varid, "scale_factor",len = scalelength))
          IF(USEL0)THEN
            call check(nf90_inq_varid(ncid, "CEFFL0", ceffl0_varid))
            call check(nf90_inquire_attribute(ncid, ceffl0_varid, "scale_factor",len = scalelength))
            call check(nf90_get_var(ncid,ceffl0_varid,LEITURACEFFL0))
            call check(nf90_inq_varid(ncid, "PEFF", peff_varid))
            call check(nf90_inquire_attribute(ncid, peff_varid, "scale_factor",len = scalelength))
            call check(nf90_get_var(ncid,peff_varid,LEITURAPEFF))
          ENDIF

          allocate(ceff_scale_factor(scalelength))
          allocate(ceffl0_scale_factor(scalelength))
          allocate(peff_scale_factor(scalelength))

          call check(nf90_get_att(ncid, ceff_varid, "scale_factor", ceff_scale_factor))
          call check(nf90_get_att(ncid, ceffl0_varid, "scale_factor", ceffl0_scale_factor))
          call check(nf90_get_att(ncid, peff_varid, "scale_factor", peff_scale_factor))

          call check(nf90_inq_dimid(ncid, LAT_NAME, lat_dimid))
          call check(nf90_inq_dimid(ncid, LON_NAME, lon_dimid))
          call check(nf90_inquire_dimension(ncid,lat_dimid,len=ilat))
          call check(nf90_inquire_dimension(ncid,lon_dimid,len=ilon))
          call check(nf90_get_var(ncid,ceff_varid,LEITURACEFF))
          call check(nf90_close(ncid) )
        ELSE
          WRITE(*,*)'Effective cloud index file not found: ', trim(CFILE)
        END IF

        DO J=1,JRANGE
          DO I=1,IRANGE
            XIMG(I,J) = LEITURACEFF(I,J)*ceff_scale_factor(1)
            IF(USEL0)THEN
              CEFF(I,J) = LEITURACEFFL0(I,J)*ceffl0_scale_factor(1)
              PEFF(I,J) = LEITURAPEFF(I,J)*peff_scale_factor(1)
            ENDIF
          ENDDO
        ENDDO
      ENDIF

          !***************************************************************
        chunk = 1
        !$OMP PARALLEL DO PRIVATE(I,J) schedule(dynamic,chunk)

        DO J=1,JRANGE
          DO I=1,IRANGE
            IF(XTCR(I,J).GE.0.0D0) THEN
              !......................................................................
              !     subroutine GLOD8A - global irradiation calculation
              IF (RUN_CS_ONLY) THEN
                CALL GLOD8A(NDIA,LON(I),LAT(J),XHOR, XTCR(I,J), XTCD(I,J),GLOBAL(I,J),XXKT(I,J),COSZEN(I,J))
              ELSE
                IF(USEL0)THEN
                    CALL GLOD8A(NDIA,LON(I),LAT(J),XHOR, XTCR(I,J), XTCD(I,J),GLOBAL(I,J),XXKT(I,J),COSZEN(I,J),CEFF=CEFF(I,J),PEFF=PEFF(I,J),TCDL0=XTCDL0(I,J))
                ELSE
                    CALL GLOD8A(NDIA,LON(I),LAT(J),XHOR, XTCR(I,J), XTCD(I,J),GLOBAL(I,J),XXKT(I,J),COSZEN(I,J),CEFF=XIMG(I,J))
                ENDIF
              ENDIF
              IF (RUN_CS_ONLY) THEN
                CALL DIFFUSE1(GLOBAL(I,J), XXKT(I,J),COSZEN(I,J),XTDIR(I,J), DIF(I,J), DIR(I,J))
                IF (OUTPUT_SCALED_DNI) THEN
                  CALL DIFFUSE1(GLOBAL(I,J), XXKT(I,J),COSZEN(I,J),XTDIRS(I,J), DIF_SCALED(I,J), DIR_SCALED(I,J))
                ENDIF
              ELSE
                CALL DIFFUSE1(GLOBAL(I,J), XXKT(I,J),COSZEN(I,J),XTDIR(I,J), DIF(I,J), DIR(I,J), XIMG(I,J))
                IF (OUTPUT_SCALED_DNI) THEN
                  CALL DIFFUSE1(GLOBAL(I,J), XXKT(I,J),COSZEN(I,J),XTDIRS(I,J), DIF_SCALED(I,J), DIR_SCALED(I,J), XIMG(I,J))
                ENDIF
              ENDIF
              IF(DIR(I,J).LT.0.0) DIR(I,J) = 0.0
              IGLO(I,J)=INT(GLOBAL(I,J))
              IDIF(I,J)=INT(DIF(I,J))
              IDIR(I,J)=INT(DIR(I,J))
              IF (OUTPUT_SCALED_DNI) THEN
                IDIF_SCALED(I,J)=INT(DIF_SCALED(I,J))
                IDIR_SCALED(I,J)=INT(DIR_SCALED(I,J))
              ENDIF
              XKT(I,J)=XXKT(I,J)
              IF(XXKT(I,J) .LT. 0.0) THEN
                 XKD(I,J)=XXKT(I,J)
              ELSE
                 XKD(I,J)=DIF(I,J)*XXKT(I,J)/GLOBAL(I,J)
                 IF (XKD(I,J) > XKT(I,J)) XKD(I,J) = XKT(I,J)
              ENDIF
              IKT(I,J)=idnint(XKT(I,J)*1000.0D0)
              IKD(I,J)=IDNINT(XKD(I,J)*1000.0D0)

            ELSEIF ((XTCR(I,J).GE.-1.01D0).AND.(XTCR(I,J).LT.-0.99D0)) THEN

              IGLO(I,J)=-1000
              IDIF(I,J)=-1000
              IDIR(I,J)=-1000
              IF (OUTPUT_SCALED_DNI) THEN
                IDIF_SCALED(I,J)=-1000
                IDIR_SCALED(I,J)=-1000
              ENDIF
              IKT(I,J) =-1000
              IKD(I,J) =-1000

            ELSEIF ((XTCR(I,J).GE.-2.01D0).AND.(XTCR(I,J).LT.-1.99D0)) THEN

              IGLO(I,J)=-2000
              IDIF(I,J)=-2000
              IDIR(I,J)=-2000
              IF (OUTPUT_SCALED_DNI) THEN
                IDIF_SCALED(I,J)=-2000
                IDIR_SCALED(I,J)=-2000
              ENDIF
              IKT(I,J) =-2000
              IKD(I,J) =-2000

            ELSE
              WRITE(*,*) 'PROBLEMAS COM A TRANSMITANCIA',i,j, XTCR(I,J)
              ! STOP

            ENDIF
          END DO !i
        END DO !j

        !$OMP END PARALLEL DO
        IF (.NOT.RUN_CS_ONLY) THEN
          deallocate(ceff_scale_factor)
          deallocate(ceffl0_scale_factor)
          deallocate(peff_scale_factor)
        END IF
        ! salvando arquivos

        ktdir=TRIM(CDIR)//CYEAR//'/'//CMON//'/images/kt'//CL0CEFF//'_'//CLMAX//TRANS_LABEL
        kddir=TRIM(CDIR)//CYEAR//'/'//CMON//'/images/kd'//CL0CEFF//'_'//CLMAX//TRANS_LABEL
        solardir=TRIM(CDIR)//CYEAR//'/'//CMON//'/images/solar_nc'//CL0CEFF//'_'//CLMAX//TRANS_LABEL

        inquire( DIRECTORY=trim(ktdir), exist=dirExists )
        IF (.NOT.(dirExists)) THEN
            result = MAKEDIRQQ(trim(ktdir))
            IF (result) THEN
               WRITE (*,*) trim(ktdir), ' successfully created'
            ELSE
               WRITE (*,*) 'Failed to create',trim(ktdir)
            END IF
        ENDIF

        inquire( DIRECTORY=trim(ktdir)//'/day'//JDAYREAD, exist=dirExists2 )
        IF (.NOT.(dirExists2)) THEN
            result = MAKEDIRQQ(trim(ktdir)//'/day'//JDAYREAD)
            IF (result) THEN
               WRITE (*,*) trim(ktdir)//'/day'//JDAYREAD, ' successfully created'
            ELSE
               WRITE (*,*) 'Failed to create',trim(ktdir)//'/day'//JDAYREAD
            END IF
        ENDIF

        CKT=trim(ktdir)//'/day'//JDAYREAD//'/kt'//JDAYREAD//CYEAR//'.'//CHORI//'.nc'
        call check(nf90_create(CKT,nf90_clobber,ncid))
        print*,CKT

        ! Define as dimens�es.
        call check( nf90_def_dim(ncid,LAT_NAME, NLAT, lat_dimid))
        call check( nf90_def_dim(ncid,LON_NAME, NLON, lon_dimid))
        call check( nf90_def_var(ncid, LAT_NAME, NF90_REAL, lat_dimid, lat_varid) )
        call check( nf90_def_var(ncid, LON_NAME, NF90_REAL, lon_dimid, lon_varid) )
        call check( nf90_put_att(ncid, lat_varid, UNITS, LAT_UNITS) )
        call check( nf90_put_att(ncid, lon_varid, UNITS, LON_UNITS) )
        dimids = (/ lon_dimid, lat_dimid /)
        call check( nf90_def_var(ncid, KT_NAME, NF90_INT, dimids, KT_varid))
        call check( nf90_put_att(ncid, KT_varid, UNITS, KT_UNITS) )
        call check( nf90_put_att(ncid, KT_varid,   OFFSET, offset_factor))
        call check( nf90_put_att(ncid, KT_varid,   SCALE, scale_factor))
        call check( nf90_put_att(ncid, NF90_GLOBAL, VERSION_NAME, VERSION_RESUME) )
        call check( nf90_enddef(ncid) )
        call check( nf90_put_var(ncid, lat_varid, LAT) )
        call check( nf90_put_var(ncid, lon_varid, LON) )
        call check( nf90_put_var(ncid, KT_varid, IKT) )
        call check( nf90_close(ncid) )

        ! salvando arquivos
        inquire( DIRECTORY=trim(kddir), exist=dirExists )
        IF (.NOT.(dirExists)) THEN
            result = MAKEDIRQQ(trim(kddir))
            IF (result) THEN
               WRITE (*,*) trim(kddir), ' successfully created'
            ELSE
               WRITE (*,*) 'Failed to create',trim(kddir)
            END IF
        ENDIF

        inquire( DIRECTORY=trim(kddir)//'/day'//JDAYREAD, exist=dirExists2 )
        IF (.NOT.(dirExists2)) THEN
            result = MAKEDIRQQ(trim(kddir)//'/day'//JDAYREAD)
            IF (result) THEN
               WRITE (*,*) trim(kddir)//'/day'//JDAYREAD, ' successfully created'
            ELSE
               WRITE (*,*) 'Failed to create',trim(kddir)//'/day'//JDAYREAD
            END IF
        ENDIF

        CKD=trim(kddir)//'/day'//JDAYREAD//'/kd'//JDAYREAD//CYEAR//'.'//CHORI//'.nc'
        ! print*,CKD
        call check(nf90_create(CKD,nf90_clobber,ncid))

        ! Define as dimens�es.
        call check( nf90_def_dim(ncid,LAT_NAME, NLAT, lat_dimid))
        call check( nf90_def_dim(ncid,LON_NAME, NLON, lon_dimid))
        call check( nf90_def_var(ncid, LAT_NAME, NF90_REAL, lat_dimid, lat_varid) )
        call check( nf90_def_var(ncid, LON_NAME, NF90_REAL, lon_dimid, lon_varid) )
        call check( nf90_put_att(ncid, lat_varid, UNITS, LAT_UNITS) )
        call check( nf90_put_att(ncid, lon_varid, UNITS, LON_UNITS) )
        dimids = (/ lon_dimid, lat_dimid /)
        call check( nf90_def_var(ncid, KD_NAME, NF90_INT, dimids, KD_varid))
        call check( nf90_put_att(ncid, KD_varid, UNITS, KD_UNITS) )
        call check( nf90_put_att(ncid, KD_varid,   OFFSET, offset_factor))
        call check( nf90_put_att(ncid, KD_varid,   SCALE, scale_factor))
        call check( nf90_put_att(ncid, NF90_GLOBAL, VERSION_NAME, VERSION_RESUME) )
        call check( nf90_enddef(ncid) )
        call check( nf90_put_var(ncid, lat_varid, LAT) )
        call check( nf90_put_var(ncid, lon_varid, LON) )
        call check( nf90_put_var(ncid, KD_varid, IKD) )
        call check( nf90_close(ncid) )

        ! salvando arquivos
        inquire( DIRECTORY=trim(solardir), exist=dirExists )
        IF (.NOT.(dirExists)) THEN
            result = MAKEDIRQQ(trim(solardir))
            IF (result) THEN
               WRITE (*,*) trim(solardir), ' successfully created'
            ELSE
               WRITE (*,*) 'Failed to create',trim(ktdir)
            END IF
        ENDIF

        inquire( DIRECTORY=trim(solardir)//'/day'//JDAYREAD, exist=dirExists2 )
        IF (.NOT.(dirExists2)) THEN
            result = MAKEDIRQQ(trim(solardir)//'/day'//JDAYREAD)
            IF (result) THEN
               WRITE (*,*) trim(solardir)//'/day'//JDAYREAD, ' successfully created'
            ELSE
               WRITE (*,*) 'Failed to create',trim(solardir)//'/day'//JDAYREAD
            END IF
        ENDIF

        arq_nc=trim(solardir)//'/day'//JDAYREAD//'/solar_nc'//JDAYREAD//CYEAR//'.'//CHORI//'.nc'
        ! print*,arq_nc

        print *, arq_nc

        call check(nf90_create(arq_nc,nf90_clobber,ncid))

        ! Define as dimens�es.
        call check( nf90_def_dim(ncid,LAT_NAME, NLAT, lat_dimid))
        call check( nf90_def_dim(ncid,LON_NAME, NLON, lon_dimid))

        ! Define as vari�veis de coordenadas. Manter�o as informa��es das coordenadas.
        ! Sendo que uma "varid" � retornada para cada.
        call check( nf90_def_var(ncid, LAT_NAME, NF90_REAL, lat_dimid, lat_varid) )
        call check( nf90_def_var(ncid, LON_NAME, NF90_REAL, lon_dimid, lon_varid) )

        ! Alinha as unidades aos dados das coordenadas, isso para cada uma.
        call check( nf90_put_att(ncid, lat_varid, UNITS, LAT_UNITS) )
        call check( nf90_put_att(ncid, lon_varid, UNITS, LON_UNITS) )

        ! Define as vari�veis netCDF para cobertuna de nuvens.
        dimids = (/ lon_dimid, lat_dimid /)
        call check( nf90_def_var(ncid, GLO_NAME, NF90_INT, dimids, glo_varid))
        call check( nf90_def_var(ncid, DIF_NAME, NF90_INT, dimids, dif_varid))
        call check( nf90_def_var(ncid, DNI_NAME, NF90_INT, dimids, dir_varid))
        IF (OUTPUT_SCALED_DNI) THEN
          call check( nf90_def_var(ncid, DIF_SCALED_NAME, NF90_INT, dimids, dif_scaled_varid))
          call check( nf90_def_var(ncid, DIR_SCALED_NAME, NF90_INT, dimids, dir_scaled_varid))
        ENDIF
        ! call check( nf90_def_var(ncid, PAR_NAME, NF90_INT, dimids, par_varid))

        ! Alia �s vari�veis as unidades correspondentes.
        call check( nf90_put_att(ncid, glo_varid, UNITS, GLO_UNITS) )
        call check( nf90_put_att(ncid, dif_varid, UNITS, DIF_UNITS) )
        call check( nf90_put_att(ncid, dir_varid, UNITS, DIR_UNITS) )

        call check( nf90_put_att(ncid, glo_varid,   OFFSET, offset_factor))
        call check( nf90_put_att(ncid, glo_varid,   SCALE, scale_factor))
        call check( nf90_put_att(ncid, dif_varid,   OFFSET, offset_factor))
        call check( nf90_put_att(ncid, dif_varid,   SCALE, scale_factor))
        call check( nf90_put_att(ncid, dir_varid,   OFFSET, offset_factor))
        call check( nf90_put_att(ncid, dir_varid,   SCALE, scale_factor))
        IF (OUTPUT_SCALED_DNI) THEN
          call check( nf90_put_att(ncid, dif_scaled_varid,   OFFSET, offset_factor))
          call check( nf90_put_att(ncid, dif_scaled_varid,   SCALE, scale_factor))
          call check( nf90_put_att(ncid, dir_scaled_varid,   OFFSET, offset_factor))
          call check( nf90_put_att(ncid, dir_scaled_varid,   SCALE, scale_factor))
        ENDIF

        call check( nf90_put_att(ncid, NF90_GLOBAL, VERSION_NAME, VERSION_RESUME) )

        ! Fim do modo de defini��o.
        call check( nf90_enddef(ncid) )

        ! Escreve os dados das vari�veis de coordenadas.
        ! Coloca as latitudes e longituides no grid do arquivo netCDF.
        call check( nf90_put_var(ncid, lat_varid, LAT) )
        call check( nf90_put_var(ncid, lon_varid, LON) )

        ! Escreve os outros dados pretendidos.
        ! Isto escrever� os dados de cobertuna de nuvens.
        ! A lista de dados s�o do mesmo tamanho que as vari�veis netCDF foram definidas.

        call check( nf90_put_var(ncid, glo_varid, IGLO) )
        call check( nf90_put_var(ncid, dif_varid, IDIF) )
        call check( nf90_put_var(ncid, dir_varid, IDIR) )
        IF (OUTPUT_SCALED_DNI) THEN
          call check( nf90_put_var(ncid, dif_scaled_varid, IDIF_SCALED) )
          call check( nf90_put_var(ncid, dir_scaled_varid, IDIR_SCALED) )
        ENDIF
        ! Fecha o arquivo.
        ! Isso faz com que o netCDF limpe toda a mem�ria tempor�ria usada at� agora e
        ! d� a certeza de que os dados s�o realmente escritos no disco.
        call check( nf90_close(ncid) )
      endif

   END DO



 CLOSE(10)

 1001 FORMAT(A180)
 1005 FORMAT(5(1X,F8.2))
!*************************************************************************
!   Mensagens de erro de leitura
 STOP
 1110 PRINT*, 'Erro de leitura do arquivo de latitude'
 STOP
 1111 PRINT*, 'Erro de leitura do arquivo de longitude'
 STOP
 1112 PRINT*, 'Erro de leitura do arquivo de imagem'
 contains
!--------------------------------------------
 subroutine check(status)
   integer, intent (in) :: status
   if(status /= nf90_noerr) then
     print *, trim(nf90_strerror(status))
     stop 2
   end if
 end subroutine check
 END
!*************************************************
 ! SUBROUTINE MOVEBITES (READING, L, C)
 ! use gradevars, only: IRANGE,JRANGE
 ! INTEGER*2 READING(IRANGE,JRANGE)
 ! INTEGER*2 ITEMP
 ! INTEGER L, C ! image counts
 ! CHARACTER*1 JTEMP(2)
 ! CHARACTER*1 KTEMP
 !
 ! EQUIVALENCE(JTEMP(1), ITEMP)
 ! SAVE
 !
 ! DO N = 1, L
 !  DO M = 1, C
 !   ITEMP = READING(M,N)
 !   KTEMP = JTEMP(2)
 !   JTEMP(2) = JTEMP(1)
 !   JTEMP(1) = KTEMP
 !   READING(M,N)= ITEMP
 !
 !  END DO
 ! END DO
 ! RETURN
 ! END SUBROUTINE
