!-------------------------------------------------------------------------------
! LABREN: Laboratório de Modelagem e Estudos de Recursos Renováveis de Energia
!-------------------------------------------------------------------------------
! SUBROUTINE: DAY
!
!> Calculates the Ordinal date [1-366], sometimes called Julian day, equivalent
!! to a Gregorian date.
!! Works by calling JULDAY to calculate the Julian day for the input date and
!! the Julian day corresponding to the first day of that year, then subtracting
!! one from the other.
!!
!! @param[in]     IMON   INTEGER Month.
!! @param[in]     IDAY   INTEGER Day of the month.
!! @param[in]     IYEAR  INTEGER Gregorian year.
!! @param[out]    JDAY   INTEGER Ordinal date [1-366] equivalent to the input
!!                               Gregorian date.
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
      SUBROUTINE DAY(IMON,IDAY,IYEAR,JDAY)

      IMPLICIT NONE
      INTERFACE
        INTEGER FUNCTION JULDAY(MM,ID,IYYY)
          INTEGER, INTENT(IN)::MM,ID,IYYY
        END FUNCTION JULDAY
      END INTERFACE
      INTERFACE
        SUBROUTINE MONTH(CMON,IMON)
          CHARACTER*3 ,intent(in)::CMON
          INTEGER,intent(out):: IMON
        END SUBROUTINE MONTH
      END INTERFACE
!     J1D   : characterisc number of the day
!     J0D   : characterisc initial number of the day in the year
!......................................................................
      INTEGER, INTENT(IN)::IMON,IDAY,IYEAR
      INTEGER, INTENT(OUT)::JDAY
      INTEGER::J1D,J0D


!     calculate de number of the day and the initial day using the
!     auxiliary function JULDAY

      J1D  = JULDAY(IMON,IDAY,IYEAR)
      J0D  = JULDAY(1,1,IYEAR)-1
      JDAY = J1D-J0D

      RETURN
    END SUBROUTINE DAY

!-------------------------------------------------------------------------------
! FUNCTION: JULDAY
!
!> Auxiliary function to calculate the Julian day equivalent to a Gregorian
!! date.
!!
!! @param[in]     MM     INTEGER Month.
!! @param[in]     ID     INTEGER Day of the month.
!! @param[in]     IYYYY  INTEGER Gregorian year.
!!
!! @return        REAL  Julian day corresponding to that day, month, and year.
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------

    INTEGER FUNCTION JULDAY(MM,ID,IYYY)

      INTEGER, INTENT(IN)::MM,ID,IYYY
      INTEGER::JY,JM,JA
      INTEGER, PARAMETER:: IGREG=15+31*(10+12*1582)

      IF (IYYY.EQ.0) write(*,*)'There is no Year Zero.',IYYY !PAUSE 'There is no Year Zero.'
      IF (MM.GT.2) THEN
        JY=IYYY
        JM=MM+1
      ELSE
        JY=IYYY-1
        JM=MM+13
      ENDIF
      JULDAY=INT(365.25*JY)+INT(30.6001*JM)+ID+1720995
      IF (ID+31*(MM+12*IYYY).GE.IGREG) THEN
        JA=INT(0.01*JY)
        JULDAY=JULDAY+2-JA+INT(0.25*JA)
      ENDIF

      RETURN
    END FUNCTION JULDAY

!-------------------------------------------------------------------------------
! SUBROUTINE: MONTH
!
!> Returns the month number based on its name, represented by a three character
!! string in English.
!!
!! @param[in]     CMON   CHARACTER*3 First three characters of the month name
!!                                   in English.
!! @param[out]    IMON   INTEGER     Month [1-12].
!!
!! @todo       Convert string to lower case before comparison to cover all
!!             cases.
!! @todo       Replace 'if' statements with a select case.
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
      SUBROUTINE MONTH(CMON,IMON)

      IMPLICIT NONE
      CHARACTER*3 ,intent(in)::CMON
      INTEGER,intent(out):: IMON

      IF((CMON.EQ.'JAN').OR.(CMON.EQ.'Jan').OR.(CMON.EQ.'jan')) IMON=1
      IF((CMON.EQ.'FEB').OR.(CMON.EQ.'Feb').OR.(CMON.EQ.'feb')) IMON=2
      IF((CMON.EQ.'MAR').OR.(CMON.EQ.'Mar').OR.(CMON.EQ.'mar')) IMON=3
      IF((CMON.EQ.'APR').OR.(CMON.EQ.'Apr').OR.(CMON.EQ.'apr')) IMON=4
      IF((CMON.EQ.'MAY').OR.(CMON.EQ.'May').OR.(CMON.EQ.'may')) IMON=5
      IF((CMON.EQ.'JUN').OR.(CMON.EQ.'Jun').OR.(CMON.EQ.'jun')) IMON=6
      IF((CMON.EQ.'JUL').OR.(CMON.EQ.'Jul').OR.(CMON.EQ.'jul')) IMON=7
      IF((CMON.EQ.'AUG').OR.(CMON.EQ.'Aug').OR.(CMON.EQ.'aug')) IMON=8
      IF((CMON.EQ.'SEP').OR.(CMON.EQ.'Sep').OR.(CMON.EQ.'sep')) IMON=9
      IF((CMON.EQ.'OCT').OR.(CMON.EQ.'Oct').OR.(CMON.EQ.'oct')) IMON=10
      IF((CMON.EQ.'NOV').OR.(CMON.EQ.'Nov').OR.(CMON.EQ.'nov')) IMON=11
      IF((CMON.EQ.'DEC').OR.(CMON.EQ.'Dec').OR.(CMON.EQ.'dec')) IMON=12

      RETURN
    END SUBROUTINE MONTH

!-------------------------------------------------------------------------------
! SUBROUTINE: DIACAR
!
!> Returns the Ordinal date (or Julian day) for which transmittance will be
!! calculated based on the month.
!! Should return a day roughly in the middle of that month [10-20].
!!
!! @param[in]     IMON     INTEGER Month.
!! @param[in]     IYEAR    INTEGER Year.
!! @param[out]    NDIACAR  INTEGER Ordinal date roughly in the middle of they
!!                                 input month.
!!
!! @todo       Replace 'if' statements with a select case.
!!
!! @warning    IYEAR is never used. This should be removed.
!!
!! @author     LABREN
!! @version    2.0
!! @date       2019
!-------------------------------------------------------------------------------
      SUBROUTINE DIACAR(IMON,IYEAR,NDIACAR)

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      INTEGER IMON,NDIA

      IF(IMON.EQ.1) NDIACAR=17
      IF(IMON.EQ.2) NDIACAR=45
      IF(IMON.EQ.3) NDIACAR=74
      IF(IMON.EQ.4) NDIACAR=105
      IF(IMON.EQ.5) NDIACAR=135
      IF(IMON.EQ.6) NDIACAR=161
      IF(IMON.EQ.7) NDIACAR=199
      IF(IMON.EQ.8) NDIACAR=230
      IF(IMON.EQ.9) NDIACAR=261
      IF(IMON.EQ.10) NDIACAR=292
      IF(IMON.EQ.11) NDIACAR=322
      IF(IMON.EQ.12) NDIACAR=347

      RETURN
      END
