!-------------------------------------------------------------------------------
! LABREN: Laboratory of Modeling and Studies on Renewable Energy Resources
! http://labren.ccst.inpe.br/index-en.html
!-------------------------------------------------------------------------------
! PROGRAM: BRASILSR_transmittances
!> Calulates global, diffuse and direct transmittances for a domain in a given day
!! and spectral irradiances for given grid points.
!!
!! Domain configuration, grid points with local data, grid points to save spectral output
!! and model configuration options can be found at the configvars module (see configvars directory in base dir)
!!
!! A full description of the model and benchmark statistics can be found at:
!!
!! Assessment of the impact of biomass burning aerosols in spectral and broadband
!! downward surface solar irradiance - a case study in Brazilian Amazon and Cerrado.
!! Madeleine S.G. Casagrande, Fernando R. Martins, Nilton M.E. Rosário, Francisco J.L. de Lima,
!! André R. Gonçalves, Rodrigo S. Costa, Antonio Maurício Zarzur, Marcelo P. Pes and Enio B. Pereira.
!! Submitted to Remote Sensing on Semptember 25th, 2021.
!!
!! @author     LABREN
!! @version    2.0
!! @date       2021
!-------------------------------------------------------------------------------
PROGRAM BRASILSR_transmittances
  use netcdf
  use omp_lib
  use gradevars
  use Brasilsr_interface
  use Brasilsr_spectral

  CHARACTER*3 CMON,CCHORI,JDAY,JDAYREAD
  CHARACTER*4 CYEAR,CHORI,CL0CEFF,CLMAX
  CHARACTER*120 CDIR,CDESC,CSAT,CANAL
  CHARACTER*250 CTRANS,CTRANSIJ
  CHARACTER*250 CIMAGEM,NCDF,CALTI,AODfile,transdir,points_data_file,points_save_file
  CHARACTER*300 AODaeronetfile,O3aeronetfile,PWVaeronetfile, EAaeronetfile
  CHARACTER*250 Biomafile, O3file, brdfile,brdfdir
  CHARACTER(256) :: my_errmsg
  CHARACTER *20, dimension(:), allocatable :: LL, LLAOD
  DOUBLE PRECISION XLAT(JRANGE),XLON(IRANGE)
  DOUBLE PRECISION XHOR,IHORI
  DOUBLE PRECISION TCLEAR(IRANGE,JRANGE),TCLOUD(IRANGE,JRANGE),TCLOUDL0(IRANGE,JRANGE)
  DOUBLE PRECISION TDIR(IRANGE,JRANGE),TDIRscaled(IRANGE,JRANGE),TTCLEAR,TTDIR,TTTDIR, cosZ
  INTEGER :: IYEAR,chunk,NDIACAR, timeshour
  INTEGER, dimension(IRANGE,JRANGE) ::XBIOMA
  DOUBLE PRECISION :: XALT(IRANGE,JRANGE)
  DOUBLE PRECISION XTEM(IRANGE,JRANGE),XUMI(IRANGE,JRANGE),XWV(IRANGE,JRANGE),XO3(IRANGE,JRANGE), XAER_EA(IRANGE,JRANGE)
  DOUBLE PRECISION, dimension(:,:,:), allocatable :: xfiso,xfgeo,xfvol
  CHARACTER *1::lk
  INTEGER :: i,j,NC,NL,NLC,ICOLI,ILINI,IMON
  INTEGER :: lat_varid, lon_varid,ilat,ilon,ilataod,ilonaod,lataod_varid, lonaod_varid,kext_varid
  INTEGER :: temp_varid,umid_varid,wv_varid,m_aod_varid,m_ea_varid, bioma_varid, o3_varid
  INTEGER :: bnlat,bnlon,fiso_varid,fvol_varid,fgeo_varid,filelenght
  INTEGER :: DIR_varid, DIRS_varid, CLEAR_varid, CLOUD_varid, CLOUDL0_varid
  INTEGER :: ncid,aodncid,biomancid,o3ncid,ncbrdfid
  INTEGER :: JJ,JJJ,K,KK,ip,ik
  INTEGER :: ndims_in, nvars_in, ngatts_in, unlimdimid_in
  INTEGER*2 LEITURAALT(NLONGI,NLATI)
  INTEGER*2 IHOR(Ntimes)
  LOGICAL RUN4IHOR(Ntimes)
  INTEGER IERRO, IERR
  INTEGER :: sub_i,sub_j,sub_k

  CHARACTER  (len = *), PARAMETER :: UNITS     = "units"
  CHARACTER  (len = *), PARAMETER :: LAT_NAME  = "latitude"
  CHARACTER  (len = *), PARAMETER :: LON_NAME  = "longitude"
  CHARACTER  (len = *), PARAMETER :: LAT_UNITS = "degrees_north"
  CHARACTER  (len = *), PARAMETER :: LON_UNITS = "degrees_east"
  CHARACTER  (len = *), PARAMETER :: UMI_NAME  = "umidade"
  CHARACTER  (len = *), PARAMETER :: TEMP_NAME  = "temperatura"
  CHARACTER  (len = *), PARAMETER :: ALB_NAME  = "albedo"
  CHARACTER  (len = *), PARAMETER :: WV_NAME  = "precipitavel"
  CHARACTER  (len = *), PARAMETER :: BIOMA_NAME  = "bioma"
  CHARACTER  (len = *), PARAMETER :: O3_NAME  = "total_ozone_column"
  CHARACTER  (len = *), PARAMETER :: FISO_NAME  = "FISO_MEAN"
  CHARACTER  (len = *), PARAMETER :: FGEO_NAME  = "FGEO_MEAN"
  CHARACTER  (len = *), PARAMETER :: FVOL_NAME  = "FVOL_MEAN"
  CHARACTER  (len = *), PARAMETER :: DIR_NAME  = "TDIR"
  CHARACTER  (len = *), PARAMETER :: DIRS_NAME  = "TDIR_scaled"
  CHARACTER  (len = *), PARAMETER :: CLEAR_NAME   = "TCLEAR"
  CHARACTER  (len = *), PARAMETER :: CLOUD_NAME   = "TCLOUD"
  CHARACTER  (len = *), PARAMETER :: CLOUDL0_NAME   = "TCLOUD_at_L0"
  CHARACTER  (len = *), PARAMETER :: DIR_UNITS  = ""
  CHARACTER  (len = *), PARAMETER :: CLEAR_UNITS  = ""
  CHARACTER  (len = *), PARAMETER :: CLOUD_UNITS  = ""
  CHARACTER  (len = *), PARAMETER :: CLOUDL0_UNITS  = ""
  CHARACTER  (len = *), PARAMETER :: MERRA_AOD_NAME = "TOTEXTTAU"
  CHARACTER  (len = *), PARAMETER :: MERRA_EA_NAME = "TOTANGSTR"

  INTEGER :: lat_dimid, lon_dimid,lataod_dimid, lonaod_dimid
  INTEGER(kind=4),dimension(2) :: dimids
  INTEGER, PARAMETER :: NLAT = JRANGE, NLON = IRANGE

  INTEGER, dimension(:,:), allocatable ::bioma, O3
  INTEGER, dimension(:,:), allocatable :: fiso, fgeo, fvol
  INTEGER  ::scalelength,blat_dimid, blon_dimid,blat_varid,blon_varid
  DOUBLE PRECISION, dimension(:), allocatable:: fiso_scale_factor, fgeo_scale_factor, fvol_scale_factor
  DOUBLE PRECISION, dimension(:,:), allocatable :: temp,umid,wvapor!,visi,albe
  DOUBLE PRECISION, dimension(:),   allocatable :: lats, lons
  DOUBLE PRECISION, dimension(:,:,:), ALLOCATABLE :: AOD550, AER_EA
  DOUBLE PRECISION, dimension(:), ALLOCATABLE:: LATAOD, LONAOD
  DOUBLE PRECISION, dimension(:,:, :), allocatable :: Kext
  LOGICAL :: fileExists, dirExists, dirExists2
  REAL time_begin, time_end
  INTEGER :: start(3), count(3)
  DOUBLE PRECISION, dimension(Ntimes)::AODaeronet
  LOGICAL scalingvar
  LOGICAL(4) result
  TYPE(point_withdata_defs), DIMENSION(num_points) :: aeronetpoint
  TYPE(point_2save_defs), DIMENSION(run_points) :: savedpoint


  CALL CPU_TIME ( time_begin )


  IF(USELMAX)THEN
    WRITE(CLMAX,'(I4)') LMAX
  ELSE
    CLMAX='orig'
  ENDIF

  IF(USEL0)THEN
    WRITE(CL0CEFF,'(I4)') L0CEFF
    PRINT*,'Running with L0 = ',CL0CEFF, ' and corresponding TAUWL0 = ', TAUWL0
  ELSE
    CL0CEFF='noL0'
    PRINT*,'Running without L0, LMAX = ',CLMAX
  ENDIF

!.....input from description parameters
  inquire( FILE='gkss.dat', exist=fileExists )
    IF (fileExists) THEN
    OPEN(UNIT=10,FILE='gkss.dat')
    DO
        READ(10,1001,IOSTAT=IERR) CDIR
        IF (IERR /= 0) EXIT
        READ(10,*) CMON,IYEAR,JDAYREAD
        WRITE(CYEAR,'(I4)') IYEAR
    END DO
    CLOSE(10)
  ELSE
    WRITE(*,*)'ERROR: Configuration file not found: gkss.dat'
    STOP
  ENDIF

  IF (USEAERONETIJ) THEN
      points_data_file=TRIM(CDIR)//CYEAR//'/points_withdata_latlon.csv'
      inquire( FILE=trim(points_data_file), exist=fileExists )
      IF (fileExists) THEN
        WRITE(*,*)'Reading info for points with local data: ',trim(points_data_file)
        OPEN(UNIT=21,FILE=points_data_file)
        DO k=1, num_points
         READ(21,*) aeronetpoint(k)%NOME, aeronetpoint(k)%LATI, aeronetpoint(k)%LONGI, aeronetpoint(k)%ALTI
        ENDDO
        CLOSE(21)
      ELSE
        WRITE(*,*)'WARNING: File with info for points with local data not found: ',trim(points_data_file)
      ENDIF
  END IF

  IF (SAVEIJ) THEN
      points_save_file=TRIM(CDIR)//CYEAR//'/points_2save_latlon.csv'
      inquire( FILE=trim(points_save_file), exist=fileExists )
      IF (fileExists) THEN
        WRITE(*,*)'Reading info for points where spectral output will be saved: ',trim(points_save_file)
        OPEN(UNIT=22,FILE=points_save_file)
        DO k=1, run_points
         READ(22,*) savedpoint(k)%NOME, savedpoint(k)%LATI, savedpoint(k)%LONGI, savedpoint(k)%ALTI
        ENDDO
        CLOSE(22)
      ELSE
        WRITE(*,*)'WARNING: File with info for points where spectral output will be saved not found: ',points_save_file,'. No data will be saved this way.'
      ENDIF
  END IF

  RUN4IHOR=.FALSE.

  timeshour=(60/IntFREQUENCY)
  DO i=8, 22
    DO j=1, timeshour
      pos=(i-8)*timeshour+j
      IHOR(pos)=i*100 + (j-1)*IntFREQUENCY
    END DO
  END DO
  JJJ=Ntimes+1

  IF (RUN_ALL_HOURS) THEN
      RUN4IHOR=.TRUE.
      write(*,*)'Running for all timesteps in day'
      JDAY=JDAYREAD
  ELSE !Not running in clear sky mode, setting RUN4IHOR to .TRUE. when satellite image is available
      CIMAGEM='imagens'//CMON//CYEAR//'.dat'
      inquire( FILE=trim(CIMAGEM), exist=fileExists )
      IF (fileExists) THEN
        WRITE(*,*)'Reading times for output from: ',trim(CIMAGEM)
        OPEN(UNIT=14,FILE=CIMAGEM)
        JJJ=1
        DO
           READ(14,*,IOSTAT=IERRO) JDAY,IYEAR,IHORI
           IF (IERRO /= 0) EXIT

           IF (USEDIARIO .AND. (JDAY.NE.JDAYREAD)) THEN
             PRINT*, 'Previously running for day ',JDAYREAD,', now detecting day ', JDAY
             STOP 'ERROR: Trying to run with data for different days with option USEDIARIO = .TRUE.'
           END IF

           JJ=JJJ
           DO KK=JJ,Ntimes
             ! PRINT*,KK, JJ, IHOR(KK), int(IHORI)
             IF(int(IHORI).EQ.IHOR(KK)) THEN
               RUN4IHOR(KK)=.TRUE.
               JJJ=JJ+1
               JJ=JJ+1
               EXIT
              ELSEIF(int(IHORI).LT.IHOR(KK)) THEN
                JJ=JJ+1
                CONTINUE
              ELSEIF(int(IHORI).EQ.IHOR(KK)) THEN
                 EXIT
              ENDIF
           END DO
        END DO
        CLOSE (14)
      ELSE
        WRITE(*,*)'ERROR: File  with times for output not found: ',trim(CIMAGEM)
        STOP
      ENDIF
  ENDIF

  CALL MONTH(CMON,IMON)
  IF (USEDIARIO) THEN
    READ(JDAYREAD,'(I3)')NDIACAR
  ELSE
    CALL DIACAR(IMON,IYEAR,NDIACAR)
  ENDIF

  NL=NLATI
  NC=NLONGI
  NLC = 2*NL*NC

  !!!.....input from ij local files
  IF (USEAERONETIJ) THEN
    DO ip=1, num_points
      AODaeronetfile=TRIM(CDIR)//CYEAR//'/'//CMON//'/prop/AOD550_'//TRIM(aeronetpoint(ip)%NOME)//'_'//CYEAR//'_'//JDAYREAD//'_'//FREQUENCY//'mins.csv'
      O3aeronetfile=TRIM(CDIR)//CYEAR//'/'//CMON//'/prop/O3_'//TRIM(aeronetpoint(ip)%NOME)//'_'//CYEAR//'_'//JDAYREAD//'_'//FREQUENCY//'mins.csv'
      PWVaeronetfile=TRIM(CDIR)//CYEAR//'/'//CMON//'/prop/PWV_'//TRIM(aeronetpoint(ip)%NOME)//'_'//CYEAR//'_'//JDAYREAD//'_'//FREQUENCY//'mins.csv'
      EAaeronetfile=TRIM(CDIR)//CYEAR//'/'//CMON//'/prop/EA_'//TRIM(aeronetpoint(ip)%NOME)//'_'//CYEAR//'_'//JDAYREAD//'_'//FREQUENCY//'mins.csv'

      inquire(file=AODaeronetfile, exist=fileExists )
      IF (fileExists) THEN
        WRITE(*,*)'Opening ',AODaeronetfile
        OPEN(UNIT=30,FILE=AODaeronetfile,IOSTAT=IERR)
        IF (IERR /= 0) THEN
          PRINT*,'Error opening file'
          EXIT
        ENDIF
        ! read(30,*)AODaeronet(1)
        DO ik=1, Ntimes
          READ(30,*) aeronetpoint(ip)%AOD550(ik)
        ENDDO
        CLOSE(30)
      ELSE
        WRITE(*,*)'WARNING: File '//TRIM(AODaeronetfile)//' not found'
        aeronetpoint(ip)%AOD550=-99
      ENDIF

      inquire( file=EAaeronetfile, exist=fileExists )
      IF (fileExists) THEN
        WRITE(*,*)'Opening ',EAaeronetfile
        OPEN(UNIT=31,FILE=EAaeronetfile,IOSTAT=IERR)
        IF (IERR /= 0) THEN
          PRINT*,'Error opening file'
          EXIT
        ENDIF
        DO ik=1, Ntimes
          READ(31,*) aeronetpoint(ip)%EA(ik)
        ENDDO
        CLOSE(31)
      ELSE
        WRITE(*,*)'WARNING: File '//TRIM(EAaeronetfile)//' not found'
        aeronetpoint(ip)%EA=-99
      ENDIF

      inquire( file=O3aeronetfile, exist=fileExists )
      IF (fileExists) THEN
        WRITE(*,*)'Opening ',O3aeronetfile
        OPEN(UNIT=32,FILE=O3aeronetfile,IOSTAT=IERR)
        IF (IERR /= 0) THEN
          PRINT*,'Error opening file'
          EXIT
        ENDIF

        DO ik=1, Ntimes
          READ(32,*) aeronetpoint(ip)%O3(ik)
        ENDDO
        CLOSE(32)
      ELSE
        WRITE(*,*)'WARNING: File '//TRIM(O3aeronetfile)//' not found'
        aeronetpoint(ip)%O3=-99
      ENDIF

      inquire( file=PWVaeronetfile, exist=fileExists )
      IF (fileExists) THEN
        WRITE(*,*)'Opening ',PWVaeronetfile
        OPEN(UNIT=33,FILE=PWVaeronetfile,IOSTAT=IERR)
        IF (IERR /= 0) THEN
          PRINT*,'Error opening file'
          EXIT
        ENDIF

        DO ik=1, Ntimes
          READ(33,*) aeronetpoint(ip)%PWV(ik)
        ENDDO
        CLOSE(33)
      ELSE
        WRITE(*,*)'WARNING: File '//TRIM(PWVaeronetfile)//' not found'
        aeronetpoint(ip)%PWV=-99
      ENDIF
    ENDDO
  ENDIF

 !**************************************************************************
 filelenght=len(trim(CDIR))-6
 brdfdir=CDIR(1:filelenght)
 PRINT*,'Reading brdf data at directory: ',trim(brdfdir)
 IF (USE_SPECTRAL_ALB) THEN
     IF (.NOT. ALLOCATED(xfiso)) ALLOCATE(xfiso(IRANGE,JRANGE,8), STAT = IERR, errmsg=my_errmsg)
       IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)
     IF (.NOT. ALLOCATED(xfgeo)) ALLOCATE(xfgeo(IRANGE,JRANGE,8), STAT = IERR, errmsg=my_errmsg)
       IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)
     IF (.NOT. ALLOCATED(xfvol)) ALLOCATE(xfvol(IRANGE,JRANGE,8), STAT = IERR, errmsg=my_errmsg)
       IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)

     DO k=1,7
        WRITE(lk,'(I1)') k
        brdfile = trim(brdfdir)//'/brdf/CDO/BRDF_parameters_Band'//lk//'_'//CMON//'.nc'
        inquire( file=trim(brdfile), exist=fileExists )
        IF (fileExists) THEN
            CALL check(nf90_open(brdfile, nf90_nowrite, ncbrdfid) )
            CALL check(nf90_inquire(ncbrdfid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))
            ALLOCATE(LL(nvars_in))

            DO i=1,nvars_in
               CALL check(nf90_inquire_variable(ncbrdfid,i,name = LL(i)))
               IF (LL(i).EQ.LAT_NAME) THEN
                 CALL check(nf90_inq_varid(ncbrdfid, LL(i), blat_varid))
                 CALL check(nf90_inq_dimid(ncbrdfid, LAT_NAME, blat_dimid))
                 CALL check(nf90_inquire_dimension(ncbrdfid,blat_dimid,len=bnlat))
               ELSE IF (LL(i).EQ.LON_NAME) THEN
                 CALL check(nf90_inq_varid(ncbrdfid, LL(i), blon_varid))
                 CALL check(nf90_inq_dimid(ncbrdfid, LON_NAME, blon_dimid))
                 CALL check(nf90_inquire_dimension(ncbrdfid,blon_dimid,len=bnlon))
               ELSE IF (LL(i).EQ.FISO_NAME) THEN
                 CALL check(nf90_inq_varid(ncbrdfid, LL(i), fiso_varid))
                 CALL check(nf90_inquire_attribute(ncbrdfid, fiso_varid, "scale_factor",len = scalelength))
               ELSE IF (LL(i).EQ.FGEO_NAME) THEN
                 CALL check(nf90_inq_varid(ncbrdfid, LL(i), fgeo_varid))
                 CALL check(nf90_inquire_attribute(ncbrdfid, fgeo_varid, "scale_factor",len = scalelength))
               ELSE IF (LL(i).EQ.FVOL_NAME) THEN
                 CALL check(nf90_inq_varid(ncbrdfid, LL(i), fvol_varid))
                 CALL check(nf90_inquire_attribute(ncbrdfid, fvol_varid, "scale_factor",len = scalelength))
               ENDIF
            ENDDO

            ALLOCATE(fiso_scale_factor(scalelength))
            ALLOCATE(fgeo_scale_factor(scalelength))
            ALLOCATE(fvol_scale_factor(scalelength))

            CALL check(nf90_get_att(ncbrdfid, fiso_varid, "scale_factor",fiso_scale_factor))
            CALL check(nf90_get_att(ncbrdfid, fgeo_varid, "scale_factor",fgeo_scale_factor))
            CALL check(nf90_get_att(ncbrdfid, fvol_varid, "scale_factor",fvol_scale_factor))

            ALLOCATE(fiso(bnlon, bnlat))
            ALLOCATE(fgeo(bnlon, bnlat))
            ALLOCATE(fvol(bnlon, bnlat))

            CALL check(nf90_get_var(ncbrdfid,fiso_varid,fiso))
            CALL check(nf90_get_var(ncbrdfid,fgeo_varid,fgeo))
            CALL check(nf90_get_var(ncbrdfid,fvol_varid,fvol))
            CALL check(nf90_close(ncbrdfid))


            sub_i=1
            sub_j=1
            DO j=JMIN,JMAX
               DO i=IMIN,IMAX
                 xfiso(sub_i,sub_j,k) = DBLE(fiso(i,j))* fiso_scale_factor(1)
                 xfgeo(sub_i,sub_j,k) = DBLE(fgeo(i,j))* fgeo_scale_factor(1)
                 xfvol(sub_i,sub_j,k) = DBLE(fvol(i,j))* fvol_scale_factor(1)
                 sub_i = sub_i + 1
               ENDDO!j
               sub_j = sub_j + 1
               sub_i=1
            ENDDO!i
            xfiso(:,:,8)=0.D0
            xfgeo(:,:,8)=0.D0
            xfvol(:,:,8)=0.D0
            DEALLOCATE(fiso)
            DEALLOCATE(fgeo)
            DEALLOCATE(fvol)
            DEALLOCATE(fiso_scale_factor)
            DEALLOCATE(fgeo_scale_factor)
            DEALLOCATE(fvol_scale_factor)
            DEALLOCATE(LL)
        ELSE
          WRITE(*,*)'ERROR: BRDF File '//trim(brdfile)//' not found'
          STOP
        ENDIF
     ENDDO
 ELSE
     IF (.NOT. ALLOCATED(xfiso)) ALLOCATE(xfiso(IRANGE,JRANGE,1), STAT = IERR, errmsg=my_errmsg)
       IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)
     IF (.NOT. ALLOCATED(xfgeo)) ALLOCATE(xfgeo(IRANGE,JRANGE,1), STAT = IERR, errmsg=my_errmsg)
       IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)
     IF (.NOT. ALLOCATED(xfvol)) ALLOCATE(xfvol(IRANGE,JRANGE,1), STAT = IERR, errmsg=my_errmsg)
       IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)

     brdfile = trim(brdfdir)//'/brdf/CDO/BRDF_parameters_shortwave_'//CMON//'.nc'
     INQUIRE( file=trim(brdfile), exist=fileExists )
     IF (fileExists) THEN
        !------------	Open netcdf com dados BDRF
        CALL check(nf90_open(brdfile, nf90_nowrite, ncbrdfid) )
        CALL check(nf90_inquire(ncbrdfid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))
        ALLOCATE(LL(nvars_in))

        DO i=1,nvars_in
           CALL check(nf90_inquire_variable(ncbrdfid,i,name = LL(i)))
           IF (LL(i).EQ.LAT_NAME) THEN
             CALL check(nf90_inq_varid(ncbrdfid, LL(i), blat_varid))
             CALL check(nf90_inq_dimid(ncbrdfid, LAT_NAME, blat_dimid))
             CALL check(nf90_inquire_dimension(ncbrdfid,blat_dimid,len=bnlat))
           ELSE IF (LL(i).EQ.LON_NAME) THEN
             CALL check(nf90_inq_varid(ncbrdfid, LL(i), blon_varid))
             CALL check(nf90_inq_dimid(ncbrdfid, LON_NAME, blon_dimid))
             CALL check(nf90_inquire_dimension(ncbrdfid,blon_dimid,len=bnlon))
           ELSE IF (LL(i).EQ.FISO_NAME) THEN
             CALL check(nf90_inq_varid(ncbrdfid, LL(i), fiso_varid))
             CALL check(nf90_inquire_attribute(ncbrdfid, fiso_varid, "scale_factor",len = scalelength))
           ELSE IF (LL(i).EQ.FGEO_NAME) THEN
             CALL check(nf90_inq_varid(ncbrdfid, LL(i), fgeo_varid))
             CALL check(nf90_inquire_attribute(ncbrdfid, fgeo_varid, "scale_factor",len = scalelength))
           ELSE IF (LL(i).EQ.FVOL_NAME) THEN
             CALL check(nf90_inq_varid(ncbrdfid, LL(i), fvol_varid))
             CALL check(nf90_inquire_attribute(ncbrdfid, fvol_varid, "scale_factor",len = scalelength))
           ENDIF
        ENDDO

        ALLOCATE(fiso_scale_factor(scalelength))
        ALLOCATE(fgeo_scale_factor(scalelength))
        ALLOCATE(fvol_scale_factor(scalelength))

        CALL check(nf90_get_att(ncbrdfid, fiso_varid, "scale_factor",fiso_scale_factor))
        CALL check(nf90_get_att(ncbrdfid, fgeo_varid, "scale_factor",fgeo_scale_factor))
        CALL check(nf90_get_att(ncbrdfid, fvol_varid, "scale_factor",fvol_scale_factor))

        ALLOCATE(fiso(bnlon, bnlat))
        ALLOCATE(fgeo(bnlon, bnlat))
        ALLOCATE(fvol(bnlon, bnlat))

        CALL check(nf90_get_var(ncbrdfid,fiso_varid,fiso))
        CALL check(nf90_get_var(ncbrdfid,fgeo_varid,fgeo))
        CALL check(nf90_get_var(ncbrdfid,fvol_varid,fvol))
        CALL check(nf90_close(ncbrdfid))

        sub_i=1
        sub_j=1
        DO j=JMIN,JMAX
           DO i=IMIN,IMAX
             xfiso(sub_i,sub_j,1) = DBLE(fiso(i,j))* fiso_scale_factor(1)
             xfgeo(sub_i,sub_j,1) = DBLE(fgeo(i,j))* fgeo_scale_factor(1)
             xfvol(sub_i,sub_j,1) = DBLE(fvol(i,j))* fvol_scale_factor(1)
             sub_i = sub_i + 1
           ENDDO!j
           sub_j = sub_j + 1
           sub_i=1
        ENDDO!i

        DEALLOCATE(fiso)
        DEALLOCATE(fgeo)
        DEALLOCATE(fvol)
        DEALLOCATE(fiso_scale_factor)
        DEALLOCATE(fgeo_scale_factor)
        DEALLOCATE(fvol_scale_factor)
        DEALLOCATE(LL)
     ENDIF
 ENDIF
 WRITE(*,*) "BRDF data processed"


 !**************************************************************************

!     LEITURA DO ARQUIVO DE BIOMA

Biomafile='/dados/estudo_caso_ICEM/bioma/biomas_goes13.nc'
inquire( file=trim(Biomafile), exist=fileExists )
IF (fileExists) THEN
    WRITE(*,*) 'Trying opening '//TRIM(Biomafile)
    CALL check(nf90_open(Biomafile, nf90_nowrite, biomancid) )
    CALL check(nf90_inquire(biomancid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))
    ALLOCATE(LL(nvars_in))

    DO i=1,nvars_in
       CALL check(nf90_inquire_variable(biomancid,i,name = LL(i)))
       IF (LL(i).EQ.LAT_NAME) THEN
         CALL check(nf90_inq_dimid(biomancid, LAT_NAME, lat_dimid))
         CALL check(nf90_inquire_dimension(biomancid,lat_dimid,len=ilat))
       ELSE IF (LL(i).EQ.LON_NAME) THEN
         CALL check(nf90_inq_dimid(biomancid, LON_NAME, lon_dimid))
         CALL check(nf90_inquire_dimension(biomancid,lon_dimid,len=ilon))
       ELSE IF (LL(i).EQ.BIOMA_NAME) THEN
         CALL check(nf90_inq_varid(biomancid, LL(i), bioma_varid))
       ENDIF
    ENDDO

    IF (.NOT. ALLOCATED(temp)) ALLOCATE(bioma(ilon, ilat), STAT = IERR, errmsg=my_errmsg)
      IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)
    CALL check(nf90_get_var(biomancid,bioma_varid,bioma))
    IF (ALLOCATED(ll)) DEALLOCATE(LL, STAT = IERR, errmsg=my_errmsg)
      IF (IERR .GT. 0) CALL handler_dealloc(IERR,my_errmsg)
    CALL check(nf90_close(biomancid))
    WRITE(*,*) 'Done'
ELSE
    WRITE(*,*)'ERROR: Bioma File '//trim(Biomafile)//' not found.'
    STOP
ENDIF

!     LEITURA DO ARQUIVO DE Ozônio

O3file=TRIM(CDIR)//CYEAR//'/'//CMON//'/prop/'//CMON//CYEAR//'_O3.nc'
inquire( file=trim(Biomafile), exist=fileExists )
IF (fileExists) THEN
    WRITE(*,*) 'Trying opening '//TRIM(O3file)
    CALL check(nf90_open(O3file, nf90_nowrite, o3ncid) )
    CALL check(nf90_inquire(o3ncid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))
    ALLOCATE(LL(nvars_in))

    DO i=1,nvars_in
       CALL check(nf90_inquire_variable(o3ncid,i,name = LL(i)))
       IF (LL(i).EQ.LAT_NAME) THEN
         CALL check(nf90_inq_dimid(o3ncid, LAT_NAME, lat_dimid))
         CALL check(nf90_inquire_dimension(o3ncid,lat_dimid,len=ilat))
       ELSE IF (LL(i).EQ.LON_NAME) THEN
         CALL check(nf90_inq_dimid(o3ncid, LON_NAME, lon_dimid))
         CALL check(nf90_inquire_dimension(o3ncid,lon_dimid,len=ilon))
       ELSE IF (LL(i).EQ.O3_NAME) THEN
         CALL check(nf90_inq_varid(o3ncid, LL(i), o3_varid))
       ENDIF
    ENDDO

    IF (.NOT. ALLOCATED(temp)) ALLOCATE(O3(ilon, ilat), STAT = IERR, errmsg=my_errmsg)
      IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)
    CALL check(nf90_get_var(o3ncid,o3_varid,O3))
    IF (ALLOCATED(ll)) DEALLOCATE(LL, STAT = IERR, errmsg=my_errmsg)
      IF (IERR .GT. 0) CALL handler_dealloc(IERR,my_errmsg)
    CALL check(nf90_close(o3ncid))
    WRITE(*,*) 'Done'
ELSE
    WRITE(*,*)'ERROR: Ozone File '//trim(O3file)//' not found.'
    STOP
ENDIF
!     LEITURA DOS ARQUIVOS DE LATITUDE E LONGITUDE CLIMATICOS e Altitude

  CALTI=TRIM(CDIR)//CYEAR//'/'//CMON//'/prop/'//CMON//CYEAR//'.alt'
  inquire( file=trim(CALTI), exist=fileExists )
  IF (fileExists) THEN
      WRITE(*,*) 'Trying opening altitude file'//TRIM(CALTI)
      OPEN(20,FILE=CALTI,FORM='unformatted',RECL=NLC,STATUS='OLD',ACCESS='direct')
      READ(20,REC=1) ((LEITURAALT(I,J),I=1,NC),J=1,NL)
      CALL MOVEBITES(LEITURAALT,NL,NC)
      ! XALT = LEITURA1/1.0     ! altitude entra em metros
      CLOSE (20)
      WRITE(*,*) 'Done'
  ELSE
      WRITE(*,*)'ERROR: Altitude File '//trim(CALTI)//' not found.'
      STOP
  ENDIF

  IF (USEDIARIO) THEN
    NCDF=TRIM(CDIR)//CYEAR//'/'//CMON//'/prop/'//CMON//'_'//CYEAR//'_'//JDAY//'_gfs.nc'
  ELSE
    NCDF=TRIM(CDIR)//CYEAR//'/'//CMON//'/prop/'//CMON//'_'//CYEAR//'_gfs.nc'
  ENDIF
  inquire( file=trim(NCDF), exist=fileExists )
  IF (fileExists) THEN
      WRITE(*,*) 'Trying opening climatic data file'//TRIM(NCDF)
      CALL check(nf90_open(NCDF, nf90_nowrite, ncid) )
      CALL check(nf90_inquire(ncid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))
      ALLOCATE(LL(nvars_in))

      DO i=1,nvars_in
         CALL check(nf90_inquire_variable(ncid,i,name = LL(i)))
         IF (LL(i).EQ.LAT_NAME) THEN
           CALL check(nf90_inq_varid(ncid, LL(i), lat_varid))
           CALL check(nf90_inq_dimid(ncid, LAT_NAME, lat_dimid))
           CALL check(nf90_inquire_dimension(ncid,lat_dimid,len=ilat))
         ELSE IF (LL(i).EQ.LON_NAME) THEN
           CALL check(nf90_inq_varid(ncid, LL(i), lon_varid))
           CALL check(nf90_inq_dimid(ncid, LON_NAME, lon_dimid))
           CALL check(nf90_inquire_dimension(ncid,lon_dimid,len=ilon))
         ELSE IF (LL(i).EQ.TEMP_NAME) THEN
           CALL check(nf90_inq_varid(ncid, LL(i), temp_varid))
         ELSE IF (LL(i).EQ.UMI_NAME) THEN
           CALL check(nf90_inq_varid(ncid, LL(i), umid_varid))
         ! ELSE IF (LL(i).EQ.ALB_NAME) THEN
         !   CALL check(nf90_inq_varid(ncid, LL(i), albe_varid))
         ELSE IF (LL(i).EQ.WV_NAME) THEN
           CALL check(nf90_inq_varid(ncid, LL(i), wv_varid))
         ENDIF
      ENDDO

      IF (.NOT. ALLOCATED(temp)) ALLOCATE(temp(ilon, ilat), STAT = IERR, errmsg=my_errmsg)
        IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)
      IF (.NOT. ALLOCATED(umid)) ALLOCATE(umid(ilon, ilat), STAT = IERR, errmsg=my_errmsg)
        IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)
      ! IF (.NOT. ALLOCATED(albe)) ALLOCATE(albe(ilon, ilat), STAT = IERR, errmsg=my_errmsg)
      !   IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)
      IF (.NOT. ALLOCATED(wvapor)) ALLOCATE(wvapor(ilon, ilat), STAT = IERR, errmsg=my_errmsg)
        IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)

      IF(.NOT. ALLOCATED(lats)) ALLOCATE(lats(ilat), STAT = IERR, errmsg=my_errmsg)
        IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)
      IF(.NOT. ALLOCATED(lons)) ALLOCATE(lons(ilon), STAT = IERR, errmsg=my_errmsg)
        IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)

      CALL check(nf90_get_var(ncid,lat_varid,lats))
      CALL check(nf90_get_var(ncid,lon_varid,lons))
      CALL check(nf90_get_var(ncid,temp_varid,temp))
      CALL check(nf90_get_var(ncid,umid_varid,umid))
      CALL check(nf90_get_var(ncid,wv_varid,wvapor))
      CALL check(nf90_close(ncid))
      WRITE(*,*) 'Done'
  ELSE
      WRITE(*,*)'ERROR: Climatic File '//trim(NCDF)//' not found.'
      STOP
  ENDIF

  IF(USEDIARIO) THEN
    IF (USEMEANAOD) THEN
      AODfile=TRIM(CDIR)//CYEAR//'/'//CMON//'/prop/MERRA2_'//CMON//CYEAR//'_'//JDAY//'_mean.nc4'
    ELSE
      AODfile=TRIM(CDIR)//CYEAR//'/'//CMON//'/prop/MERRA2_'//CMON//CYEAR//'_'//JDAY//'_'//FREQUENCY//'mins.nc4'
    ENDIF
  ELSE
    IF (USEMEANAOD) THEN
      AODfile=TRIM(CDIR)//CYEAR//'/'//CMON//'/prop/MERRA2_'//CMON//CYEAR//'_mean.nc4'
    ELSE
      AODfile=TRIM(CDIR)//CYEAR//'/'//CMON//'/prop/MERRA2_'//CMON//CYEAR//'_'//JDAY//'_mean_'//FREQUENCY//'mins.nc4'
    ENDIF
  ENDIF
  inquire( file=trim(AODfile), exist=fileExists )
  IF (fileExists) THEN
      WRITE(*,*) 'Trying reading AOD regional data from: ',TRIM(AODfile)
      CALL check(nf90_open(AODfile, nf90_nowrite, aodncid) )
      CALL check(nf90_inquire(aodncid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))
      CALL check(nf90_inq_varid(aodncid, LAT_NAME, lataod_varid))
      CALL check(nf90_inq_varid(aodncid, LON_NAME, lonaod_varid))
      CALL check(nf90_inq_varid(aodncid, MERRA_AOD_NAME, m_aod_varid))
      CALL check(nf90_inq_varid(aodncid, MERRA_EA_NAME, m_ea_varid))

      CALL check(nf90_inq_dimid(aodncid, LAT_NAME, lataod_dimid))
      CALL check(nf90_inq_dimid(aodncid, LON_NAME, lonaod_dimid))

      CALL check(nf90_inquire_dimension(aodncid,lataod_dimid,len=ilataod))
      CALL check(nf90_inquire_dimension(aodncid,lonaod_dimid,len=ilonaod))

      IF (USEMEANAOD) THEN
        IF(.NOT. ALLOCATED(AOD550)) ALLOCATE(AOD550(ilonaod, ilataod,1), STAT = IERR, errmsg=my_errmsg)
          IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)
        CALL check(nf90_get_var(aodncid,m_aod_varid, AOD550))
        IF(.NOT. ALLOCATED(AER_EA)) ALLOCATE(AER_EA(ilonaod, ilataod,1), STAT = IERR, errmsg=my_errmsg)
          IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)
        CALL check(nf90_get_var(aodncid,m_ea_varid, AER_EA))
      ELSE
        IF(.NOT. ALLOCATED(AOD550)) ALLOCATE(AOD550(ilonaod, ilataod, Ntimes), STAT = IERR, errmsg=my_errmsg)
          IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)
        CALL check(nf90_get_var(aodncid,m_aod_varid, AOD550))
        IF(.NOT. ALLOCATED(AER_EA)) ALLOCATE(AER_EA(ilonaod, ilataod, Ntimes), STAT = IERR, errmsg=my_errmsg)
          IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)
        CALL check(nf90_get_var(aodncid,m_ea_varid, AER_EA))
      ENDIF

      CALL check(nf90_close(aodncid))
      WRITE(*,*)'MERRA2 file read succesfully'
    ELSE
        WRITE(*,*)'ERROR: AOD regional File '//trim(AODfile)//' not found.'
        STOP
    ENDIF


  !!Getting location of points in grid coords
    IF (USEAERONETIJ) THEN
      DO k=1, num_points
        Call Gridpoint_ij(lats, lons, aeronetpoint(k)%LATI, aeronetpoint(k)%LONGI, aeronetpoint(k)%GRADI, aeronetpoint(k)%GRADJ)
      ENDDO
    ENDIF
    IF (SAVEIJ) THEN
      DO k=1, run_points
        Call Gridpoint_ij(lats, lons, savedpoint(k)%LATI, savedpoint(k)%LONGI, savedpoint(k)%GRADI, savedpoint(k)%GRADJ)
      ENDDO
    ENDIF


  !!!Filling data for the subdomain
    sub_i=1
    sub_j=1
    DO j=JMIN,JMAX
    	DO i=IMIN,IMAX
         XTEM(sub_i,sub_j) = temp(i,j)
    	   XUMI(sub_i,sub_j) = umid(i,j)
         !Convert to cm of precipitable water vapor
    	   XWV(sub_i,sub_j) = wvapor(i,j)/10.0_dp
         !**********************************
    	   XLON(sub_i)   = lons(i)
         XALT(sub_i,sub_j) = LEITURAALT(i,j)/1.0_dp
         XBIOMA(sub_i,sub_j) = bioma(i,j)
         XO3(sub_i,sub_j) = DBLE(O3(i,j))
         sub_i = sub_i + 1
    	ENDDO!i
      XLAT(sub_j)   = lats(j)
      sub_j = sub_j + 1
      sub_i=1
    ENDDO!j

  WRITE(*,*)"*** SUCCESS: read all data files !"

  !Deallocating data for the larger domain that is no longer required
    IF (ALLOCATED(ll)) DEALLOCATE(LL, STAT = IERR, errmsg=my_errmsg)
      IF (IERR .GT. 0) CALL handler_dealloc(IERR,my_errmsg)
    IF (ALLOCATED(temp)) DEALLOCATE(temp, STAT = IERR, errmsg=my_errmsg)
      IF (IERR .GT. 0) CALL handler_dealloc(IERR,my_errmsg)
    IF (ALLOCATED(umid)) DEALLOCATE(umid, STAT = IERR, errmsg=my_errmsg)
      IF (IERR .GT. 0) CALL handler_dealloc(IERR,my_errmsg)
    IF (ALLOCATED(wvapor)) DEALLOCATE(wvapor, STAT = IERR, errmsg=my_errmsg)
      IF (IERR .GT. 0) CALL handler_dealloc(IERR,my_errmsg)
    IF (ALLOCATED(bioma)) DEALLOCATE(bioma, STAT = IERR, errmsg=my_errmsg)
      IF (IERR .GT. 0) CALL handler_dealloc(IERR,my_errmsg)
    IF (ALLOCATED(O3)) DEALLOCATE(O3, STAT = IERR, errmsg=my_errmsg)
      IF (IERR .GT. 0) CALL handler_dealloc(IERR,my_errmsg)
    IF (ALLOCATED(lats)) DEALLOCATE(lats, STAT = IERR, errmsg=my_errmsg)
      IF (IERR .GT. 0) CALL handler_dealloc(IERR,my_errmsg)
    IF (ALLOCATED(lons)) DEALLOCATE(lons, STAT = IERR, errmsg=my_errmsg)
      IF (IERR .GT. 0) CALL handler_dealloc(IERR,my_errmsg)

  IF(.NOT. ALLOCATED(Kext)) ALLOCATE(Kext(IRANGE,JRANGE, 5), STAT = IERR, errmsg=my_errmsg)
    IF (IERR .GT. 0) CALL handler_alloc(IERR,my_errmsg)

!!! Calculating Extnction coefficient profiles when using a daily mean AOD and AE
  IF (USEMEANAOD) THEN
    IF (USEAERONETIJ) THEN
      DO ip=1, num_points
        AOD550(aeronetpoint(ip)%GRADI+IMIN,aeronetpoint(ip)%GRADJ+JMIN,1)=sum(aeronetpoint(ip)%AOD550(:))/Ntimes
        AER_EA(aeronetpoint(ip)%GRADI+IMIN,aeronetpoint(ip)%GRADJ+JMIN,1)=sum(aeronetpoint(ip)%EA(:))/Ntimes
      ENDDO
    ENDIF
    sub_i=1
    sub_j=1
    DO j=JMIN,JMAX
      DO i=IMIN,IMAX
          CALL AerExtinction(AOD550(I,J,1),XALT(sub_i,sub_j), Kext(sub_i,sub_j,:))
          XAER_EA(sub_i,sub_j) = AER_EA(i,j,1)
          sub_i = sub_i + 1
      ENDDO!j
      sub_j = sub_j + 1
      sub_i=1
    ENDDO!i
  ENDIF

  transdir=TRIM(CDIR)//CYEAR//'/'//CMON//'/trans'//CL0CEFF//'_'//CLMAX//trim(TRANS_LABEL)

  !!!Check whether parent transmittance directory exists, if not, create it
  inquire( DIRECTORY=trim(transdir), exist=dirExists )
  IF (.NOT.(dirExists)) THEN
      result = MAKEDIRQQ(trim(transdir))
      IF (result) THEN
         WRITE (*,*) trim(transdir), ' successfully created'
      ELSE
         WRITE (*,*) 'Failed to create',trim(transdir)
         STOP
      END IF
  ENDIF
  IF (USEDIARIO) THEN !A directory is created for each day
    !!!Check whether transmittance directory for this day exists, if not, create it
    inquire( DIRECTORY=trim(transdir)//'/day'//JDAYREAD, exist=dirExists2 )
    IF (.NOT.(dirExists2)) THEN
        result = MAKEDIRQQ(trim(transdir)//'/day'//JDAYREAD)
        IF (result) THEN
           WRITE (*,*) trim(transdir)//'/day'//JDAYREAD, ' successfully created'
        ELSE
           WRITE (*,*) 'Failed to create',trim(transdir)//'/day'//JDAYREAD
           STOP
        END IF
    ENDIF
  ENDIF

!!! Starting calculations for each time step
  DO K=1,NTimes
    IF (RUN4IHOR(K)) THEN

        XHOR=DINT(IHOR(K)/100.0_dp)
        XHOR=XHOR+(IHOR(K)-100.0_dp*XHOR)/60.0_dp
  !    ABERTURA DOS ARQUIVOS DE TRANSMITANCIAS

        IF(IHOR(K).LT.1000) THEN
           WRITE(CCHORI,'(I3)') IHOR(K)
           CHORI='0'//CCHORI
        ELSE
           WRITE(CHORI,'(I4)') IHOR(K)
        ENDIF

        ! print*,k,CHORI

        IF (USEDIARIO) THEN
          CTRANS=TRIM(transdir)//'/day'//JDAYREAD//'/trans'//CMON//CYEAR//JDAYREAD//'.'//CHORI//'.nc'
        ELSE
          CTRANS=TRIM(transdir)//'/trans'//CMON//CYEAR//'.'//CHORI//'.nc'
        ENDIF

        IF (USEAERONETIJ) THEN
          DO ip=1, num_points
            IF(minval(aeronetpoint(ip)%PWV(:))>=0) THEN
                XWV(aeronetpoint(ip)%GRADI,aeronetpoint(ip)%GRADJ)=aeronetpoint(ip)%PWV(k)
            ENDIF
            IF(minval(aeronetpoint(ip)%AOD550(:))>=0) THEN
              IF (.NOT.(USEMEANAOD)) THEN
                AOD550(aeronetpoint(ip)%GRADI+IMIN,aeronetpoint(ip)%GRADJ+JMIN,k)=aeronetpoint(ip)%AOD550(k)
                AER_EA(aeronetpoint(ip)%GRADI+IMIN,aeronetpoint(ip)%GRADJ+JMIN,k)=aeronetpoint(ip)%EA(k)
              ENDIF
            ENDIF
            IF(minval(aeronetpoint(ip)%O3(:))>=0) THEN
              XO3(aeronetpoint(ip)%GRADI,aeronetpoint(ip)%GRADJ)=sum(aeronetpoint(ip)%O3(:))/Ntimes
            ENDIF
          ENDDO
        ENDIF

        IF (.NOT. USEMEANAOD) THEN
          sub_i=1
          sub_j=1
          DO j=JMIN,JMAX
            DO i=IMIN,IMAX
               IF(USE_KAEXT_MAXSECOND) THEN
                 CALL AerExtinction2(AOD550(I,J,k),XALT(sub_i,sub_j), Kext(sub_i,sub_j,:))
               ELSE
                 CALL AerExtinction(AOD550(I,J,k),XALT(sub_i,sub_j), Kext(sub_i,sub_j,:))
               ENDIF
               XAER_EA(sub_i,sub_j) = AER_EA(i,j,k)
               sub_i = sub_i + 1
            ENDDO!j
            sub_j = sub_j + 1
            sub_i=1
          ENDDO!i
        ENDIF
    IF (RUN_REGIONAL) THEN
      !Abrindo arquivo NETCDF para salvar a saida
      WRITE(*,*) 'Writing output for time ',IHOR(K),' in ',trim(CTRANS)
      CALL check(nf90_create(CTRANS,nf90_clobber,ncid))


      write(*,*) 'Starting regional computations'
        !
        ! ! !-------------------------------------------------------------------------------------------
          chunk=1
          !$omp parallel do private(I,J) schedule(dynamic, chunk)
          DO J=1,JRANGE
              DO I=1,IRANGE
                IF(XALT(I,J).GT.-98.0_dp) THEN
                  CALL D2STR(NDIACAR,XLON(I),XLAT(J),XBIOMA(I,J),XTEM(I,J),xfiso(I,J,:),xfgeo(I,J,:),xfvol(I,J,:),XUMI(I,J),XWV(I,J),XO3(I,J),XALT(I,J), &
                    & Kext(I,J,1:5),XHOR,0, 0.0_dp,TCLEAR(I,J),TDIRscaled(I,J),cosZ,EAngstrom=XAER_EA(I,J))
                  CALL D2STR(NDIACAR,XLON(I),XLAT(J),XBIOMA(I,J),XTEM(I,J),xfiso(I,J,:),xfgeo(I,J,:),xfvol(I,J,:),XUMI(I,J),XWV(I,J),XO3(I,J),XALT(I,J), &
                  & Kext(I,J,1:5),XHOR,0, 0.0_dp,TTCLEAR,TDIR(I,J),cosZ,EAngstrom=XAER_EA(I,J), scaling=scalingvar)
                  IF(USEL0)THEN
                    CALL D2STR(NDIACAR,XLON(I),XLAT(J),XBIOMA(I,J),XTEM(I,J),xfiso(I,J,:),xfgeo(I,J,:),xfvol(I,J,:),XUMI(I,J),XWV(I,J),XO3(I,J),XALT(I,J),&
                    & Kext(I,J,1:5),XHOR,1,TAUWL0,TCLOUDL0(I,J),TTDIR,cosZ,EAngstrom=XAER_EA(I,J))
                  ENDIF
                  CALL D2STR(NDIACAR,XLON(I),XLAT(J),XBIOMA(I,J),XTEM(I,J),xfiso(I,J,:),xfgeo(I,J,:),xfvol(I,J,:),XUMI(I,J),XWV(I,J),XO3(I,J),XALT(I,J), &
                  & Kext(I,J,1:5),XHOR,1,100.0_dp, TCLOUD(I,J),TTTDIR,cosZ,EAngstrom=XAER_EA(I,J))
                ELSE
                   TCLEAR(I,J) = -1.0_dp
                   TCLOUD(I,J) = -1.0_dp
                   IF(USEL0)THEN
                     TCLOUDL0(I,J) = -1.0_dp
                   ENDIF
                   TDIR(I,J)  = -1.0_dp
                ENDIF
              END DO !235
          END DO !233
                !$omp end parallel do
        !     !-------------------------------------------------------------------------
        ! ! !  Define as dimens�es.
        !
          WRITE(*,*) 'transmittances calculated'
          CALL check( nf90_def_dim(ncid,LAT_NAME, JRANGE, lat_dimid))
          CALL check( nf90_def_dim(ncid,LON_NAME, IRANGE, lon_dimid))
          CALL check( nf90_def_var(ncid, LAT_NAME, NF90_DOUBLE, lat_dimid, lat_varid) )
          CALL check( nf90_def_var(ncid, LON_NAME, NF90_DOUBLE, lon_dimid, lon_varid) )
          CALL check( nf90_put_att(ncid, lat_varid, UNITS, LAT_UNITS) )
          CALL check( nf90_put_att(ncid, lon_varid, UNITS, LON_UNITS) )
          dimids = (/ lon_dimid, lat_dimid /)

          CALL check( nf90_def_var(ncid, DIR_NAME,   NF90_DOUBLE, dimids, DIR_varid))
          CALL check( nf90_def_var(ncid, DIRS_NAME,   NF90_DOUBLE, dimids, DIRS_varid))
          CALL check( nf90_def_var(ncid, CLEAR_NAME, NF90_DOUBLE, dimids, CLEAR_varid))
          IF(USEL0)THEN
            CALL check( nf90_def_var(ncid, CLOUDL0_NAME, NF90_DOUBLE, dimids, CLOUDL0_varid))
          ENDIF
          CALL check( nf90_def_var(ncid, CLOUD_NAME, NF90_DOUBLE, dimids, CLOUD_varid))

          CALL check( nf90_put_att(ncid, DIR_varid,   UNITS, DIR_UNITS))
          CALL check( nf90_put_att(ncid, DIRS_varid,   UNITS, DIR_UNITS))
          CALL check( nf90_put_att(ncid, CLEAR_varid, UNITS, CLEAR_UNITS))
          IF(USEL0)THEN
            CALL check( nf90_put_att(ncid, CLOUDL0_varid, UNITS, CLOUDL0_UNITS) )
          ENDIF
          CALL check( nf90_put_att(ncid, CLOUD_varid, UNITS, CLOUD_UNITS) )
          CALL check( nf90_enddef(ncid))

          CALL check( nf90_put_var(ncid, lat_varid,   XLAT) )
          CALL check( nf90_put_var(ncid, lon_varid,   XLON) )

          CALL check( nf90_put_var(ncid, DIR_varid,   TDIR) )
          CALL check( nf90_put_var(ncid, DIRS_varid,   TDIRscaled) )
          CALL check( nf90_put_var(ncid, CLEAR_varid, TCLEAR) )
          IF(USEL0)THEN
            CALL check( nf90_put_var(ncid, CLOUDL0_varid, TCLOUDL0) )
          ENDIF
          CALL check( nf90_put_var(ncid, CLOUD_varid, TCLOUD) )
          CALL check( nf90_close(ncid))
        ENDIF

        !!  Calculates spectral transmittances for i,j


        IF (SAVEIJ) THEN
          write(*,*)'Saving point spectral data'
          DO ip=1, run_points

              !!!***********************************************
              !!Below, condition to avoid creating output for insitu case when there is no data, only for paper
              IF (USEAERONETIJ) THEN
                AODaeronetfile=TRIM(CDIR)//CYEAR//'/'//CMON//'/prop/AOD550_'//TRIM(savedpoint(ip)%NOME)//'_'//CYEAR//'_'//JDAYREAD//'_'//FREQUENCY//'mins.csv'
                inquire( file=AODaeronetfile, exist=fileExists )
                IF (.NOT.(fileExists)) THEN
                    print*,'Data for ',TRIM(savedpoint(ip)%NOME),' not found, skipping saving output for in_situ case study'
                    CYCLE
                ENDIF
              ENDIF
                !!!End here
              !!!***********************************************

              print*,'calculating spectral transmittance for ',TRIM(savedpoint(ip)%NOME)
              point_i=savedpoint(ip)%GRADI
              point_j=savedpoint(ip)%GRADJ
              XALT(point_i,point_j)=savedpoint(ip)%ALTI

              IF (.NOT. USEMEANAOD) THEN
                IF(USE_KAEXT_MAXSECOND) THEN
                  CALL AerExtinction2(AOD550(point_i+IMIN,point_j+JMIN,k), XALT(point_i,point_j), Kext(point_i,point_j,:))
                ELSE
                  CALL AerExtinction(AOD550(point_i+IMIN,point_j+JMIN,k), XALT(point_i,point_j), Kext(point_i,point_j,:))
                ENDIF
              ELSE
                IF(USE_KAEXT_MAXSECOND) THEN
                  CALL AerExtinction2(AOD550(point_i+IMIN,point_j+JMIN,1), XALT(point_i,point_j), Kext(point_i,point_j,:))
                ELSE
                  CALL AerExtinction(AOD550(point_i+IMIN,point_j+JMIN,1), XALT(point_i,point_j), Kext(point_i,point_j,:))
                ENDIF
              ENDIF
              write(*,*) 'Starting grid point computations'
              print*,point_i,point_j
              CTRANSIJ=TRIM(transdir)//'/day'//JDAYREAD//'/trans'//TRIM(savedpoint(ip)%NOME)//'_'//CMON//CYEAR//'_'//JDAYREAD//'.'//CHORI//'.csv'
              print*,'Saving results in ',  trim(CTRANSIJ)
              open(40,file=trim(CTRANSIJ))


                ! scaling=.TRUE.
                CALL D2STR(NDIACAR,savedpoint(ip)%LONGI,savedpoint(ip)%LATI,XBIOMA(point_i,point_j),XTEM(point_i,point_j),&
                    & xfiso(point_i,point_j,:),xfgeo(point_i,point_j,:),xfvol(point_i,point_j,:),&
                    & XUMI(point_i,point_j),XWV(point_i,point_j),XO3(point_i,point_j),XALT(point_i,point_j),&
                    & Kext(point_i,point_j,:),XHOR,0, 0.0D0,TCLEAR(point_i,point_j),TDIRscaled(point_i,point_j),cosZ, &
                    & EAngstrom=XAER_EA(point_i,point_j),ip=ip)
                scalingvar=.FALSE.
                CALL D2STR(NDIACAR,savedpoint(ip)%LONGI,savedpoint(ip)%LATI,XBIOMA(point_i,point_j),XTEM(point_i,point_j),&
                    & xfiso(point_i,point_j,:),xfgeo(point_i,point_j,:),xfvol(point_i,point_j,:),&
                    & XUMI(point_i,point_j),XWV(point_i,point_j),XO3(point_i,point_j),XALT(point_i,point_j),&
                    & Kext(point_i,point_j,:),XHOR,0, 0.0D0,TTCLEAR,TDIR(point_i,point_j),cosZ, &
                    & EAngstrom=XAER_EA(point_i,point_j),scaling=scalingvar,ip=ip)
                WRITE(40,*)'XHOR cosZ TCLEAR TDIRscaled TDIR'
                WRITE(40,'(5(f9.4))')XHOR,cosZ,TCLEAR(point_i,point_j),TDIRscaled(point_i,point_j),TDIR(point_i,point_j)
                WRITE(40,*)'dir_surf dir_scaled_surf dir_top dif_surf dif_top'
                DO i=1,MAXWEL
                  WRITE(40,'(5(f9.4))')spectral_dir_surf(i,ip),spectral_dir_scaled_surf(i,ip),spectral_dir_top(i,ip),spectral_dif_surf(i,ip),spectral_dif_top(i,ip)
                ENDDO
              CLOSE(40)
              print*,'finished'
          ENDDO
        ENDIF
    ENDIF
    !------------------------------------------------------------------------
  END DO

  IF (ALLOCATED(AOD550)) DEALLOCATE(AOD550, STAT = IERR, errmsg=my_errmsg)
    IF (IERR .GT. 0) CALL handler_dealloc(IERR,my_errmsg)
  IF (ALLOCATED(AER_EA)) DEALLOCATE(AER_EA, STAT = IERR, errmsg=my_errmsg)
    IF (IERR .GT. 0) CALL handler_dealloc(IERR,my_errmsg)
  IF (ALLOCATED(Kext)) DEALLOCATE(Kext, STAT = IERR, errmsg=my_errmsg)
    IF (IERR .GT. 0) CALL handler_dealloc(IERR,my_errmsg)

  CALL CPU_TIME ( time_end )

  PRINT *, 'Time of operation was ', time_end - time_begin, ' seconds'

 1001 FORMAT(A120)
 1003 FORMAT(A2)
 1005 FORMAT(2(1X,F8.2),3(1X,F8.3))

!*************************************************************************
      STOP
 1112 PRINT*, 'Erro de leitura do arquivo de altitude'
!*************************************************************************
 contains
!--------------------------------------------
 SUBROUTINE check(status)
   INTEGER, INTENT (in) :: status
   IF(status /= nf90_noerr) THEN
     print *, trim(nf90_strerror(status))
     stop 2
   END IF
 end SUBROUTINE check

!*************************************************
 SUBROUTINE MOVEBITES (READING, L, C)
 use gradevars
 INTEGER*2 READING(NLONGI,NLATI)
 INTEGER*2 ITEMP
 INTEGER L, C          ! image counts
 CHARACTER*1 JTEMP(2)
 CHARACTER*1 KTEMP

 EQUIVALENCE(JTEMP(1), ITEMP)
 SAVE
 DO N = 1, L
   DO M = 1, C
      ITEMP = READING(M,N)
      KTEMP = JTEMP(2)
      JTEMP(2) = JTEMP(1)
      JTEMP(1) = KTEMP
      READING(M,N)= ITEMP
   END DO
 END DO
RETURN
END SUBROUTINE
!**************************************************************************
END PROGRAM
