!-------------------------------------------------------------------------------
! LABREN: Laboratório de Modelagem e Estudos de Recursos Renováveis de Energia
!-------------------------------------------------------------------------------
! SUBROUTINE: Gridpoint_ij
!
!> Get location of grid points in positional (i,j) coordinates.
!!
!! @param[in]  lons DOUBLE, dimension(NLONGI) longitudes of domain grid
!! @param[in]  lats DOUBLE, dimension(NLATI) latitudes of domain grid
!! @param[in]  LONGI DOUBLE longitude of grid point
!! @param[in]  LATI DOUBLE latitude of grid point
!! @param[out]  GRADI INTEGER horizontal location of grid point in domain coords
!! @param[in]  GRADJ INTEGER vertical location of grid point in domain coords
!! @author     LABREN
!! @version    2.0
!! @date       2021
!-------------------------------------------------------------------------------
subroutine Gridpoint_ij(lats, lons, LATI, LONGI, GRADI, GRADJ)
 use gradevars
 use Brasilsr_constants, ONLY: dp
 IMPLICIT NONE
 DOUBLE PRECISION, dimension(NLONGI), INTENT(IN) :: lons
 DOUBLE PRECISION, dimension(NLATI), INTENT(IN) :: lats
 DOUBLE PRECISION, INTENT(IN) :: LATI, LONGI
 INTEGER, INTENT(OUT) :: GRADI, GRADJ

 INTEGER:: i, j, k
 INTEGER :: Imin_guess, Jmin_guess
 DOUBLE PRECISION :: diflat, diflon

 Imin_guess=int((LONGI-lons(1))/cellsizelon)
 Jmin_guess=int((lats(1)-LATI)/cellsizelat)

 if (abs(lons(Imin_guess)-LONGI).LE.cellsizelon/2._dp) then
   GRADI=i-Imin_guess
 elseif (abs(lons(Imin_guess+1)-LONGI).LE.abs(lons(Imin_guess-1)-LONGI)) then
   do i=Imin_guess, Imin_guess+10
     if (abs(lons(i)-LONGI).LE.cellsizelon/2._dp) then
       GRADI=i-IMIN
       EXIT
     endif
   enddo
 else
   do i=Imin_guess-10, Imin_guess
     if (abs(lons(i)-LONGI).LE.cellsizelon/2._dp) then
       GRADI=i-IMIN
       EXIT
     endif
   enddo
 endif

 if (abs(lats(Jmin_guess)-LATI).LE.cellsizelat/2._dp) then
   GRADJ=i-Jmin_guess
 elseif (abs(lats(Jmin_guess+1)-LATI).LE.abs(lats(Jmin_guess-1)-LATI)) then
   do i=Jmin_guess, Jmin_guess+10
     if (abs(lats(i)-LATI).LE.cellsizelat/2._dp) then
       GRADJ=i-JMIN
       EXIT
     endif
   enddo
 else
   do i=Jmin_guess-10, Jmin_guess
     if (abs(lats(i)-LATI).LE.cellsizelat/2._dp) then
       GRADJ=i-JMIN
       EXIT
     endif
   enddo
  endif


end subroutine Gridpoint_ij

!!--------------------------------------------
DOUBLE PRECISION function Nradius(phi)
   use Brasilsr_constants, ONLY: dp
!!Radius of curvature in the prime vertical to calculate deltalon
 double precision, intent(in)::phi !in degrees
 DOUBLE PRECISION, PARAMETER :: eq_r=6378137.0_dp, pi=3.1415927_dp,e2=0.00669437999014_dp

 Nradius=eq_r/sqrt(1-e2*(sin(phi*(pi/180.0_dp)))**2)
 RETURN
end function Nradius

!!--------------------------------------------
DOUBLE PRECISION function Mradius(phi)
   use Brasilsr_constants, ONLY: dp
!!Radius of curvature in the meridian to calculate deltalat
 double precision, intent(in)::phi
 DOUBLE PRECISION, PARAMETER :: eq_r=6378137.0_dp, pi=3.1415927_dp,e2=0.00669437999014_dp

 Mradius=eq_r/sqrt(1-e2*(sin(phi*(pi/180.0_dp)))**2)

 RETURN
end function Mradius
