!-------------------------------------------------------------------------------
! LABREN: Laboratory of Modeling and Studies on Renewable Energy Resources
! http://labren.ccst.inpe.br/index-en.html
!-------------------------------------------------------------------------------
!Program cobertura
!!@details uses minimum and maximum visible brightness for a fixed geometry and month
!! and calculates values for the day/time geometry, to determine the cloud effective index
!! and cloud effective optical depth index, if using USEL0 = .TRUE.
!!
!! A full description of the model and benchmark statistics can be found at:
!!
!! Assessment of the impact of biomass burning aerosols in spectral and broadband
!! downward surface solar irradiance - a case study in Brazilian Amazon and Cerrado.
!! Madeleine S.G. Casagrande, Fernando R. Martins, Nilton M.E. Rosário, Francisco J.L. de Lima,
!! André R. Gonçalves, Rodrigo S. Costa, Antonio Maurício Zarzur, Marcelo P. Pes and Enio B. Pereira.
!! Submitted to Remote Sensing on Semptember 25th, 2021.
!!
!! @author     LABREN
!! @version    2.0
!! @date       2021
!-------------------------------------------------------------------------------

!ifort -g -o cob_nuvens_3400_7000_brdf cobertura_nuvensNETCDF_brdf.f90 -DMKL_ILP64 -DMKL_ILP64 -I/opt/intel/mkl/include -I/opt/libraries/intel/include -I/opt/intel/include -I/opt/intel/include/intel64 -I../configvars/ -L/opt/libraries/intel/lib -L/opt/intel/lib/intel64 -lnetcdff -lnetcdf
!ifort -g -o cob_nuvens_3400_7000_brdf cobertura_nuvensNETCDF_brdf.f90 -DMKL_ILP64 -DMKL_ILP64 -I/opt/intel/mkl/include -I/opt/libs/intel/include -I/opt/intel/include -I/opt/intel/include/intel64 -I../configvars/ -L/opt/libs/intel/lib -L/opt/intel/lib/intel64 -lnetcdff -lnetcdf
program cobertura
use netcdf
use gradevars
implicit none
 character*200 dire2,dire3,arq,fmax,fmin, ncbrdf, brdfile,brdfdir
 character*200 ncdf,cob, img_netcdf,dir_netcdf
 character*200 outputPrefix
 character*30  arq_netcdf
 character*22  fnetcdf
 CHARACTER*6 CTAUWL0
 character*4   ano,hora,CL0CEFF,CLMAX
 character*3   mes,dia
 integer :: ir_varid, vi_varid, ang_sol_varid, ang_sat_varid, ang_sat_sol_varid
 integer :: minp2_varid, ceff_varid, ceffl0_varid, peff_varid, bdrf_varid, max_varid
 integer :: ndims_in, nvars_in, ngatts_in, unlimdimid_in
 integer :: lat_varid, lon_varid,ilat,ilon
 integer ::  i, j, ncid, ncidm, ncbrdfid, ncmaxid,ierr, ierr2, sub_i, sub_j, cont
 integer*2 peff(IRANGE,JRANGE), ceffl0(IRANGE,JRANGE), ceff(IRANGE,JRANGE), minP2(IRANGE,JRANGE),minimo(IRANGE,JRANGE),maximo(IRANGE,JRANGE)
 real inf,sup, infpeff, suppeff, infceffl0, supceffl0
 real, dimension(:),   allocatable :: lats, lons
 real, dimension(:,:), allocatable :: vi
 REAL*8 XLAT(JRANGE),XLON(IRANGE)
 integer*2 Xvi(IRANGE,JRANGE)

 REAL(KIND=8) :: Xvinorm(IRANGE,JRANGE), XsolarZ(IRANGE,JRANGE), XviewZ(IRANGE,JRANGE), Xxi(IRANGE,JRANGE),u0(IRANGE,JRANGE)
 REAL(KIND=8) :: xfiso(IRANGE,JRANGE),xfgeo(IRANGE,JRANGE),xfvol(IRANGE,JRANGE), fixedbrdf(IRANGE,JRANGE)
 real(KIND=8):: fixedZ, fixedV, fixedphi, fixedxi, fixedkvol, fixedkgeo
 real(KIND=8), dimension(:,:), allocatable :: solarZ, viewZ, xi
 real(KIND=8), parameter ::pi = 3.1415927

 character*20, dimension(:), allocatable :: LL
 integer :: ntime_varid,blat_varid,blon_varid,fiso_varid,fvol_varid,fgeo_varid
 integer :: bnlat,bnlon, ntime
 integer :: blat_dimid, blon_dimid, ntime_dimid
 REAL(KIND=8), dimension(:), allocatable ::blats, blons
 integer*2, dimension(:,:), allocatable :: fiso, fgeo, fvol
 integer :: numdims,numatts
 integer, dimension(3)::vardimids

 character (len = *), parameter :: BLAT_NAME  = "latitude"
 character (len = *), parameter :: BLON_NAME  = "longitude"
 character (len = *), parameter :: NTIME_NAME  = "time"
 character (len = *), parameter :: FISO_NAME  = "FISO_MEAN"
 character (len = *), parameter :: FGEO_NAME  = "FGEO_MEAN"
 character (len = *), parameter :: FVOL_NAME  = "FVOL_MEAN"
 integer              ::scalelength
 real, dimension(:), allocatable:: fiso_scale_factor, fgeo_scale_factor, fvol_scale_factor

 integer(kind=4),dimension(2) :: dimids

 character (len = *), parameter :: UNITS     = "units"
 character (len = *), parameter :: LONGNAME  = "long_name"
 character (len = *), parameter :: SCALE     = "scale_factor"
 character (len = *), parameter :: OFFSET     = "add_offset"

 character (len = *), parameter :: LAT_NAME   = "latitude"
 character (len = *), parameter :: LON_NAME   = "longitude"
 character (len = *), parameter :: LONG      = "long_name"
 character (len = *), parameter :: LAT_UNITS  = "degrees_north"
 character (len = *), parameter :: LON_UNITS  = "degrees_east"
 character (len = *), parameter :: CEFF_NAME  = "CEFF"
 character (len = *), parameter :: CEFF_LONGNAME = "Cloud cover"
 character (len = *), parameter :: CEFFL0_NAME  = "CEFFL0"
 character (len = *), parameter :: CEFFL0_LONGNAME = "Cloud cover in relation to L0"
 character (len = *), parameter :: CEFF_UNITS = " "
 character (len = *), parameter :: PEFF_NAME  = "PEFF"
 character (len = *), parameter :: PEFF_LONGNAME = "Cloud optical depth"
 character (len = *), parameter :: PEFF_UNITS = " "

 character (len = *), parameter :: VERSION_NAME  = "Cloud_cover_version"

 CHARACTER*200:: VERSION_RESUME

 DOUBLE PRECISION, PARAMETER:: offset_factor=0.D0, scale_factor=0.0001D0
 integer, parameter ::  NLAT = JRANGE, NLON = IRANGE
 integer :: lat_dimid, lon_dimid

  print*, "*** Calculando a cobertura de nuvens ***"

  call getarg(1,mes)
  call getarg(2,ano)
  ! Diretório de saída do modelo (i.e.: diretório onde 'modeloYYYY' foi criada.)
  ! Não tem '/' no final!
  call getarg(3,outputPrefix)


   WRITE(CTAUWL0,'(F6.3)') TAUWL0

   IF(USELMAX.EQ.1)THEN
     WRITE(CLMAX,'(I4)') LMAX
   ELSE
     CLMAX='orig'
   ENDIF

   IF(USEL0.EQ.1)THEN
     WRITE(CL0CEFF,'(I4)') L0CEFF
   ELSE
     CL0CEFF='noL0'
   ENDIF

   IF(USEL0.EQ.1)THEN
     VERSION_RESUME='Version feature/broad_albedo (min/ceff brdf) with L0 = '//CL0CEFF//', TAUWL0 = '//CTAUWL0//' and LMAX = '//CLMAX
   ELSE
     VERSION_RESUME='Version feature/broad_albedo (min/ceff brdf) without L0 and with LMAX = '//CLMAX
   ENDIF
   print*,VERSION_RESUME


  dir_netcdf  = trim(outputPrefix)//"/modelo"//ano//"/"//mes//"/images/netcdf/"
  dire2 = trim(outputPrefix)//"/modelo"//ano//"/"//mes//"/"
  dire3 = trim(outputPrefix)//"/modelo"//ano//"/"//mes//"/images/cn"//CL0CEFF//"_"//CLMAX//"_brdf/"
  img_netcdf = trim(outputPrefix)//"/modelo"//ano//"/"//mes//"/images/netcdf/image_horas.dat"
  brdfdir=trim(outputPrefix)//"/brdf/CDO/"

  PRINT*,'Will attempt to save results in ', TRIM(dire3)

  brdfile = 'BRDF_parameters_Band1_'//mes//'.nc'
  ncbrdf=trim(brdfdir)//trim(brdfile)
  print*,'Using BRDF file:',ncbrdf

   fixedZ=pi/4.
   fixedV=pi/6.
   fixedphi=0.
   fixedxi=acos(cos(fixedZ)*cos(fixedV) + sin(fixedZ)*sin(fixedV)*cos(fixedphi))
   fixedkgeo=Kgeo(fixedZ,fixedZ,fixedxi)
   fixedkvol=Kvol(fixedZ,fixedZ,fixedxi)

  !------------	Open netcdf com dados BDRF
  call check(nf90_open(ncbrdf, nf90_nowrite, ncbrdfid) )
  call check(nf90_inquire(ncbrdfid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))
  call check(nf90_inquire(ncbrdfid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))
  allocate(LL(nvars_in))

  do i=1,nvars_in
     call check(nf90_inquire_variable(ncbrdfid,i,name = LL(i)))
     print*,LL(i)
     if (LL(i).EQ.NTIME_NAME) then
       call check(nf90_inq_varid(ncbrdfid, LL(i), ntime_varid))
       call check(nf90_inq_dimid(ncbrdfid, NTIME_NAME, ntime_dimid))
       call check(nf90_inquire_dimension(ncbrdfid,ntime_dimid,len=ntime))
     else if (LL(i).EQ.BLAT_NAME) then
       call check(nf90_inq_varid(ncbrdfid, LL(i), blat_varid))
       call check(nf90_inq_dimid(ncbrdfid, BLAT_NAME, blat_dimid))
       call check(nf90_inquire_dimension(ncbrdfid,blat_dimid,len=bnlat))
     else if (LL(i).EQ.BLON_NAME) then
       call check(nf90_inq_varid(ncbrdfid, LL(i), blon_varid))
       call check(nf90_inq_dimid(ncbrdfid, BLON_NAME, blon_dimid))
       call check(nf90_inquire_dimension(ncbrdfid,blon_dimid,len=bnlon))
     else if (LL(i).EQ.FISO_NAME) then
       call check(nf90_inq_varid(ncbrdfid, LL(i), fiso_varid))
       call check(nf90_inquire_attribute(ncbrdfid, fiso_varid, "scale_factor",len = scalelength))
     else if (LL(i).EQ.FGEO_NAME) then
       call check(nf90_inq_varid(ncbrdfid, LL(i), fgeo_varid))
       call check(nf90_inquire_attribute(ncbrdfid, fgeo_varid, "scale_factor",len = scalelength))
     else if (LL(i).EQ.FVOL_NAME) then
       call check(nf90_inq_varid(ncbrdfid, LL(i), fvol_varid))
       call check(nf90_inquire_attribute(ncbrdfid, fvol_varid, "scale_factor",len = scalelength))
     endif
  enddo

  allocate(fiso_scale_factor(scalelength))
  allocate(fgeo_scale_factor(scalelength))
  allocate(fvol_scale_factor(scalelength))

  call check(nf90_get_att(ncbrdfid, fiso_varid, "scale_factor",fiso_scale_factor))
  call check(nf90_get_att(ncbrdfid, fgeo_varid, "scale_factor",fgeo_scale_factor))
  call check(nf90_get_att(ncbrdfid, fvol_varid, "scale_factor",fvol_scale_factor))

  allocate(blats(bnlat))
  allocate(blons(bnlon))


  call check(nf90_get_var(ncbrdfid,blat_varid,blats))
  call check(nf90_get_var(ncbrdfid,blon_varid,blons))

  allocate(fiso(bnlon, bnlat))
  allocate(fgeo(bnlon, bnlat))
  allocate(fvol(bnlon, bnlat))


  call check(nf90_get_var(ncbrdfid,fiso_varid,fiso))
  call check(nf90_get_var(ncbrdfid,fgeo_varid,fgeo))
  call check(nf90_get_var(ncbrdfid,fvol_varid,fvol))
  call check(nf90_close(ncbrdfid))

  sub_i=1
  sub_j=1
  do j=JMIN,JMAX
     do i=IMIN,IMAX
       xfiso(sub_i,sub_j) = fiso(i,j) * fiso_scale_factor(1)
       xfgeo(sub_i,sub_j) = fgeo(i,j) * fgeo_scale_factor(1)
       xfvol(sub_i,sub_j) = fvol(i,j) * fvol_scale_factor(1)
       fixedbrdf(sub_i,sub_j)=xfiso(sub_i,sub_j)+xfgeo(sub_i,sub_j)*fixedkgeo+xfvol(sub_i,sub_j)*fixedkvol
       sub_i = sub_i + 1
     enddo!j
     sub_j = sub_j + 1
     sub_i=1
  enddo!i

  deallocate(fiso)
  deallocate(fgeo)
  deallocate(fvol)
  deallocate(blats)
  deallocate(blons)
  deallocate(fiso_scale_factor)
  deallocate(fgeo_scale_factor)
  deallocate(fvol_scale_factor)
  print*, "BRDF data processed"


 !**************************************************************************
 !----------------------------------------------
  fmin = trim(dire2)//'maxmin/'//mes//ano//'_brdf_p2.nc'
  print*,'Opening P2 BRDF min in ',fmin

  call check(nf90_open(fmin, nf90_nowrite, ncid) )
  call check(nf90_inquire(ncid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))

  call check(nf90_inq_varid(ncid,"latitude", lat_varid))
  call check(nf90_inq_varid(ncid,"longitude", lon_varid))
  call check(nf90_inq_varid(ncid,"P2_brdf", minp2_varid))

  call check(nf90_inq_dimid(ncid, LAT_NAME, lat_dimid))
  call check(nf90_inq_dimid(ncid, LON_NAME, lon_dimid))

  call check(nf90_inquire_dimension(ncid,lat_dimid,len=ilat))
  call check(nf90_inquire_dimension(ncid,lon_dimid,len=ilon))

  ! call check(nf90_get_var(ncid,max_varid,maximo))
  call check(nf90_get_var(ncid, minp2_varid, minP2))

  call check(nf90_close(ncid))

 print*, "Min P2 (BRDF) data processed"

  !Abre os netcdf's
 open(25,file=img_netcdf)
 hour_loop: do
      read(25,111,iostat=ierr) fnetcdf
      111 format(22A)
      print*,'hourly imagens in ',trim(dir_netcdf)//fnetcdf
      if((ierr/=0) .OR. (len(trim(fnetcdf)).LT.22)) exit
      cont = 0
      hora = fnetcdf(15:18)
      if (USELMAX.EQ.0) then
         !READ MAXIMUM DATA FOR THE HOUR
          fmax = trim(dire2)//'maxmin/'//mes//ano//'.'//hora//'_max.nc'
          print*,fmax

          call check(nf90_open(fmax, nf90_nowrite, ncmaxid) )
          call check(nf90_inquire(ncmaxid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))

          call check(nf90_inq_varid(ncmaxid,"latitude", lat_varid))
          call check(nf90_inq_varid(ncmaxid,"longitude", lon_varid))
          call check(nf90_inq_varid(ncmaxid,"MAXIMO", max_varid))

          call check(nf90_inq_dimid(ncmaxid, LAT_NAME, lat_dimid))
          call check(nf90_inq_dimid(ncmaxid, LON_NAME, lon_dimid))

          call check(nf90_inquire_dimension(ncmaxid,lat_dimid,len=ilat))
          call check(nf90_inquire_dimension(ncmaxid,lon_dimid,len=ilon))

          call check(nf90_get_var(ncmaxid,max_varid,maximo))

          call check(nf90_close(ncmaxid))
          print*, "Maximum reflectance data processed for hour ", hora
      endif

      open(13,file=trim(dir_netcdf)//fnetcdf)
      sat_loop: do
          read(13,100,iostat=ierr2) arq_netcdf
          100 format(18A)
          print*,'sat file ',arq_netcdf
          cont = cont + 1
          if(ierr2/=0) exit
          dia  = arq_netcdf(4:6)

          ncdf=trim(dir_netcdf)//arq_netcdf

          call check(nf90_open(ncdf, nf90_nowrite, ncid) )
          call check(nf90_inquire(ncid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))

          call check(nf90_inq_varid(ncid,"latitude", lat_varid))
          call check(nf90_inq_varid(ncid,"longitude", lon_varid))
          call check(nf90_inq_varid(ncid,"imagem_ir", ir_varid))
          call check(nf90_inq_varid(ncid,"imagem_vi", vi_varid))
          call check(nf90_inq_varid(ncid,"angulo_sol", ang_sol_varid))
          call check(nf90_inq_varid(ncid,"angulo_satelite", ang_sat_varid))
          call check(nf90_inq_varid(ncid,"angulo_satelite_sol", ang_sat_sol_varid))

          call check(nf90_inq_dimid(ncid, LAT_NAME, lat_dimid))
          call check(nf90_inq_dimid(ncid, LON_NAME, lon_dimid))

          call check(nf90_inquire_dimension(ncid,lat_dimid,len=ilat))
          call check(nf90_inquire_dimension(ncid,lon_dimid,len=ilon))

          allocate(vi(ilon, ilat))

          allocate(lats(ilat))
          allocate(lons(ilon))
          allocate(solarZ(ilon, ilat))
          allocate(viewZ(ilon, ilat))
          allocate(xi(ilon, ilat))

          call check(nf90_get_var(ncid,lat_varid,lats))
          call check(nf90_get_var(ncid,lon_varid,lons))
          call check(nf90_get_var(ncid,vi_varid,vi))
          call check(nf90_get_var(ncid,ang_sol_varid,solarZ))
          call check(nf90_get_var(ncid,ang_sat_varid,viewZ))
          call check(nf90_get_var(ncid,ang_sat_sol_varid,xi))

          call check(nf90_close(ncid))

          sub_i=1
          sub_j=1
          do j=JMIN,JMAX
            do i=IMIN,IMAX
              Xvi(sub_i,sub_j) = vi(i,j)
              XsolarZ(sub_i,sub_j) = solarZ(i,j)
              XviewZ(sub_i,sub_j) = viewZ(i,j)
              Xxi(sub_i,sub_j) = xi(i,j)
              XLON(sub_i)   = lons(i)
              sub_i = sub_i + 1
            enddo
            XLAT(sub_j)   = lats(j)
            sub_j = sub_j + 1
            sub_i=1
          enddo

          deallocate(lats)
          deallocate(lons)
          deallocate(vi)
          deallocate(solarZ)
          deallocate(viewZ)
          deallocate(xi)


          XsolarZ=XsolarZ*pi/180.
          XviewZ=XviewZ*pi/180.
          Xxi=Xxi*pi/180.
          u0=cos(XsolarZ)

          if (USELMAX.EQ.1) then  !! IF USELMAX==0, maximo comes from file fmax
            maximo=u0*LMAX
          endif

          ciclo_j:do j=1,JRANGE
            ciclo_i:do i=1,IRANGE
              !This calculation is made here to use the geometry parameters of the current satellite file
              if (XsolarZ(i,j) .GE. 0.) then
                minimo(i,j) = minP2(i,j)*(brdf(xfiso(i,j), xfgeo(i,j), xfvol(i,j), XsolarZ(i,j),XviewZ(i,j),Xxi(i,j))*cos(XsolarZ(i,j)))/(fixedbrdf(i,j)*cos(fixedZ))!RECALCULATE FOR THE TIME OF DAY
              else
                minimo(i,j) =0.
              endif
              if ((minimo(i,j) .LT. 0).OR.(maximo(i,j) .LT. 0).OR.(Xvi(i,j) .LT. 0)) then !!Invalid data
                ceff(i,j) = 20000
                if (USEL0.EQ.1) then
                  ceffl0(i,j) = 20000
                  peff(i,j) = 20000
                endif
              else
                !Always calculate the original Ceff, to be used in DNI calculation
                inf = maximo(i,j) - minimo(i,j)
                if (inf .LT. 0) then !Avoid dividing by 0
                  ceff(i,j) = 20000
                else
                  sup = Xvi(i,j) - minimo(i,j)
                  if(sup .LT. 0)then ! value is lower than the P2 minimum, but it is a valid data corresponding to clear sky
                    ceff(i,j)=0
                  else
                    ceff(i,j) = (sup/inf)*10000
                    if(ceff(i,j) .GT. 10000)then
                      ceff(i,j) = 10000
                    endif
                  endif
                endif
                !Calculate Ceff e peff L0
                if (USEL0.EQ.1) then
                    if ((u0(i,j) .LT. 0.D0) .OR. (Xvi(i,j) .LT. 0.D0)) then
                      ceffl0(i,j) = 20000
                      peff(i,j) = 20000
                    elseif (Xvi(i,j) .LE. minimo(i,j)) then ! value is lower than the P2 minimum, but it is a valid data corresponding to  clear sky
                      ceffl0(i,j)=0
                      peff(i,j)=0
                    else
                        if (Xvi(i,j) .LE. L0CEFF*u0(i,j)) then !the sky is partially covered by clouds
                          infceffl0 = L0CEFF*u0(i,j) - minimo(i,j)
                          if(infceffl0 .LE. 0)then
                            ceffl0(i,j) = 20000
                            peff(i,j)=20000
                          else
                             supceffl0 = Xvi(i,j) - minimo(i,j)
                             ceffl0(i,j) = (supceffl0/infceffl0)*10000
                          endif
                          peff(i,j)=0
                        else         !the sky is fully covered by clouds
                          ceffl0(i,j) = 10000
                          infpeff = maximo(i,j)-L0CEFF*u0(i,j)
                          if(infpeff .LE. 0)then
                            peff(i,j) = 20000
                            ! print*,'WARNING, Maximum for the cell',i ,',',j ,' is lower than L0*u0', maximo(i,j),L0CEFF*u0(i,j)
                          else
                            suppeff = Xvi(i,j) - L0CEFF*u0(i,j)
                            peff(i,j) = (suppeff/infpeff)*10000
                            if(peff(i,j) .GT. 10000)then
                              peff(i,j) = 10000
                            endif
                          endif
                        endif
                    endif
                    if ((ceffl0(i,j).NE.20000).AND.(peff(i,j).NE.20000).AND.(((ceffl0(i,j)-10000)+ peff(i,j)).GT.10000 )) then
                      print*,'WARNING, ceff + peff >1', Xvi(i,j), L0CEFF*u0(i,j),minimo(i,j),maximo(i,j), ceff(i,j), peff(i,j), u0(i,j)
                    endif
                endif
              endif
            end do ciclo_i
          end do ciclo_j

    !----------------------------------------------

        cob = trim(dire3)//'cn'//dia//ano//'.'//hora//'.nc'

        call check(nf90_create(cob,nf90_clobber,ncid))

        call check( nf90_def_dim(ncid,LAT_NAME, NLAT, lat_dimid))
        call check( nf90_def_dim(ncid,LON_NAME, NLON, lon_dimid))

        call check( nf90_def_var(ncid, LAT_NAME, NF90_REAL, lat_dimid, lat_varid) )
        call check( nf90_def_var(ncid, LON_NAME, NF90_REAL, lon_dimid, lon_varid) )

        call check( nf90_put_att(ncid, lat_varid, UNITS, LAT_UNITS) )
        call check( nf90_put_att(ncid, lon_varid, UNITS, LON_UNITS) )

        dimids = (/ lon_dimid, lat_dimid /)

        call check( nf90_def_var(ncid, CEFF_NAME, NF90_INT, dimids, ceff_varid))
        call check( nf90_put_att(ncid, ceff_varid, LONG, CEFF_LONGNAME) )
        call check( nf90_put_att(ncid, ceff_varid, UNITS, CEFF_UNITS) )
        call check( nf90_put_att(ncid, ceff_varid,   OFFSET, offset_factor))
        call check( nf90_put_att(ncid, ceff_varid,   SCALE, scale_factor))

        if (USEL0.EQ.1) then
            call check( nf90_def_var(ncid, CEFFL0_NAME, NF90_INT, dimids, ceffl0_varid))
            call check( nf90_put_att(ncid, ceffl0_varid, LONG, CEFFL0_LONGNAME) )
            call check( nf90_put_att(ncid, ceffl0_varid, UNITS, CEFF_UNITS) )
            call check( nf90_put_att(ncid, ceffl0_varid,   OFFSET, offset_factor))
            call check( nf90_put_att(ncid, ceffl0_varid,   SCALE, scale_factor))
            call check( nf90_def_var(ncid, PEFF_NAME, NF90_INT, dimids, peff_varid))
            call check( nf90_put_att(ncid, peff_varid, LONG, PEFF_LONGNAME) )
            call check( nf90_put_att(ncid, peff_varid, UNITS, PEFF_UNITS) )
            call check( nf90_put_att(ncid, peff_varid,   OFFSET, offset_factor))
            call check( nf90_put_att(ncid, peff_varid,   SCALE, scale_factor))
        endif
        call check( nf90_put_att(ncid, NF90_GLOBAL, VERSION_NAME, VERSION_RESUME) )

        call check( nf90_enddef(ncid) )

        call check( nf90_put_var(ncid, lat_varid, XLAT) )
        call check( nf90_put_var(ncid, lon_varid, XLON) )

        call check( nf90_put_var(ncid, ceff_varid, ceff) )

        if (USEL0.EQ.1) then
            call check( nf90_put_var(ncid, ceffl0_varid, ceffl0) )
            call check( nf90_put_var(ncid, peff_varid, peff) )
        endif

        call check( nf90_close(ncid) )
    end do sat_loop
 end do hour_loop
 contains
!--------------------------------------------
 subroutine check(status)
   integer, intent (in) :: status
   if(status /= nf90_noerr) then
     print *, trim(nf90_strerror(status))
     stop 2
   end if
 end subroutine check
!*************************************************

  !**************************************************************************
    real function brdf(fiso, fgeo, fvol, solarZ,viewZ,xi)
      real*8, intent (in) :: fiso, fgeo, fvol, solarZ,viewZ,xi

      brdf=fiso+fgeo*Kgeo(solarZ,viewZ,xi)+fvol*Kvol(solarZ,viewZ,xi)

    end function brdf
   !*************************************************

    real function Kvol(solarZ,viewZ,xi)
      REAL(KIND=8), intent (in) :: solarZ,viewZ,xi
      REAL(KIND=8), parameter ::pi =  3.1415927

      ! print*,solarZ*180./pi,viewZ*180./pi, phi*180./pi, xi*180./pi, xi, cos(xi),cos(solarZ)*cos(viewZ) + sin(solarZ)*sin(viewZ)*cos(phi), sin(xi)
      Kvol=((pi/2.-xi)*cos(xi)+sin(xi))/(cos(solarZ)+cos(viewZ))-pi/4.
      ! print*,(pi/2.-xi),cos(xi),sin(xi),(cos(solarZ)+cos(viewZ)),Kvol
      ! pause
    end function Kvol
   !--------------------------------------------
    real function Kgeo(solarZ,viewZ,xi)
      REAL(KIND=8), intent (in) :: solarZ,viewZ,xi
      REAL(KIND=8), parameter ::pi = 3.1415927
      REAL(KIND=8), parameter :: hb=2.000000  !h/b
      REAL(KIND=8), parameter :: br=1.000000   !b/r
      REAL(KIND=8)::O,D,t,cost,phi

      phi=acos((cos(xi)-cos(solarZ)*cos(viewZ))/(sin(solarZ)*sin(viewZ)))
      D=sqrt((tan(solarZ))**2+(tan(viewZ))**2-2*tan(solarZ)*tan(viewZ)*cos(phi))
      cost=hb*sqrt(D**2+(tan(solarZ)*tan(ViewZ)*sin(phi))**2)/((1/cos(solarZ))+(1/cos(viewZ)))
      t=acos(min(max(cost,-1.),1.))
      ! if(cost.LE.1.0000)then
      !   t=acos(cost)
      ! else
      !   t=asin(1-cost)
      ! endif
      O=(1./pi)*(t-sin(t)*cos(t))*((1/cos(solarZ))+(1/cos(viewZ)))
      Kgeo=O-(1./cos(solarZ))-(1./cos(viewZ))+(1./2.)*(1+cos(xi))*(1/cos(viewZ))*(1/cos(solarZ))

    end function Kgeo


end program
