!-------------------------------------------------------------------------------
! LABREN: Laboratory of Modeling and Studies on Renewable Energy Resources
! http://labren.ccst.inpe.br/index-en.html
!-------------------------------------------------------------------------------
!Program min_max
!!@details calculates minimum and maximum visible brightness for a fixed geometry and month
!!
!! A full description of the model and benchmark statistics can be found at:
!!
!! Assessment of the impact of biomass burning aerosols in spectral and broadband
!! downward surface solar irradiance - a case study in Brazilian Amazon and Cerrado.
!! Madeleine S.G. Casagrande, Fernando R. Martins, Nilton M.E. Rosário, Francisco J.L. de Lima,
!! André R. Gonçalves, Rodrigo S. Costa, Antonio Maurício Zarzur, Marcelo P. Pes and Enio B. Pereira.
!! Submitted to Remote Sensing on Semptember 25th, 2021.
!!
!! @author     LABREN
!! @version    2.0
!! @date       2021
!-------------------------------------------------------------------------------


! ifort -o MaxMinP2brdf max_minNetcdf_p2_bdrf.f90 -DMKL_ILP64 -I/opt/intel/mkl/include -I/opt/libraries/intel/include -I/opt/intel/include -I/opt/intel/include/intel64 -I../configvars -L/opt/libraries/intel/lib -L/opt/intel/lib/intel64 -lnetcdff -lnetcdf
! ifort -o MaxMinP2brdf max_minNetcdf_p2_bdrf.f90 -DMKL_ILP64 -I/opt/intel/mkl/include -I/opt/libs/intel/include -I/opt/intel/include -I/opt/intel/include/intel64 -I/Dados/brasil-sr/configvars -L/opt/libs/intel/lib -L/opt/intel/lib/intel64 -lnetcdff -lnetcdf
program min_max
  use netcdf
  use gradevars
  implicit none
 character*200 dir_netcdf,name_netcdf
 character*200 dir_maxmin, file_saida,img_netcdf, ncbrdf, brdfile,brdfdir
 character*200 outputPrefix
 character*30  arq_netcdf
 character*22  fnetcdf
 CHARACTER*6 CTAUWL0
 character*4   hora,ano,CL0CEFF,CLMAX
 character*3   dia, mes
 integer*2     NVALID(IRANGE,JRANGE), PMIN_POS(IRANGE,JRANGE) !15 é o P2 de 740
 REAL (KIND=8) ::minimos(IRANGE,JRANGE,20), minP2(IRANGE,JRANGE),maximo(IRANGE,JRANGE)
 INTEGER(KIND=1):: ISVALID(IRANGE,JRANGE)
 integer  ::   i, j, ncid, ncidm, ncbrdfid, ierr, ierr2, sub_i, sub_j
 integer  ::   ndims_in, nvars_in, ngatts_in, unlimdimid_in
 integer  ::  lat_varid, lon_varid,ir_varid, vi_varid,ang_sol_varid,ang_sat_varid,ang_sat_sol_varid
 integer :: max_varid
 integer ::p2_varid
 integer :: lat_dimid, lon_dimid
 integer  ::  ilat,ilon
 integer  :: cont
 integer*2, dimension(:,:),   allocatable :: vi
 real(KIND=8), dimension(:,:), allocatable :: solarZ, viewZ, xi
 real(KIND=8), parameter ::pi = 3.1415927
 real(KIND=8):: fixedZ, fixedV, fixedphi, fixedxi, fixedkvol, fixedkgeo
 real(KIND=8), dimension(:),   allocatable :: lats, lons
 REAL(KIND=8):: XLAT(JRANGE),XLON(IRANGE)
 integer*2:: Xvi(IRANGE,JRANGE)
 REAL(KIND=8) :: Xvinorm(IRANGE,JRANGE), XsolarZ(IRANGE,JRANGE), XviewZ(IRANGE,JRANGE), Xxi(IRANGE,JRANGE)
 REAL(KIND=8) :: xfiso(IRANGE,JRANGE),xfgeo(IRANGE,JRANGE),xfvol(IRANGE,JRANGE), fixedbrdf(IRANGE,JRANGE)
 INTEGER(KIND=2) :: maxpos(1)

 character (len = *), parameter :: UNITS     = "units"
 character (len = *), parameter :: LONGNAME  = "long_name"
 character (len = *), parameter :: SCALE     = "scale_factor"
 character (len = *), parameter :: OFFSET     = "add_offset"

 character (len = *), parameter :: BLAT_NAME  = "latitude"
 character (len = *), parameter :: BLON_NAME  = "longitude"
 character (len = *), parameter :: NTIME_NAME  = "time"
 character (len = *), parameter :: FISO_NAME  = "FISO_MEAN"
 character (len = *), parameter :: FGEO_NAME  = "FGEO_MEAN"
 character (len = *), parameter :: FVOL_NAME  = "FVOL_MEAN"

 character*20, dimension(:), allocatable :: LL
 integer :: ntime_varid,blat_varid,blon_varid,fiso_varid,fvol_varid,fgeo_varid
 integer :: bnlat,bnlon, ntime
 integer :: blat_dimid, blon_dimid, ntime_dimid
 REAL(KIND=8), dimension(:), allocatable ::blats, blons
 REAL(KIND=8) , dimension(:,:), allocatable :: fiso, fgeo, fvol
  ! integer :: numdims,numatts
 ! integer, dimension(3)::vardimids

 character (len = *), parameter :: LAT_NAME  = "latitude"
 character (len = *), parameter :: LON_NAME  = "longitude"
 character (len = *), parameter :: LAT_UNITS = "degrees_north"
 character (len = *), parameter :: LON_UNITS = "degrees_east"
 character (len = *), parameter :: MAX_NAME  = "MAXIMO"
 character (len = *), parameter :: MAX_LONGNAME   = "Maximum reflectance among values of the same hour and month"
 character (len = *), parameter :: MAX_UNITS  = " "
 character (len = *), parameter :: P2_NAME  = "P2_brdf"
 character (len = *), parameter :: P2_LONGNAME   = "P2 reflectance of values with  Z<60, normalized using BRDF"
 character (len = *), parameter :: P2_UNITS  = " "

 character (len = *), parameter :: VERSION_NAME  = "MaxMin_version"

 CHARACTER*200:: VERSION_RESUME
 integer              ::scalelength
 real, dimension(:), allocatable:: fiso_scale_factor, fgeo_scale_factor, fvol_scale_factor
 integer, parameter :: NLAT = JRANGE, NLON = IRANGE
 DOUBLE PRECISION, PARAMETER:: offset_factor=0.D0, scale_factor=1.D0
 integer(kind=4),dimension(2) :: dimids

 LOGICAL VALIDMASK(IRANGE,JRANGE), satexists
 integer :: percentil

 NVALID=0
 minimos = 32767
 maximo= -32767
 Xvinorm=32767
 fixedZ=pi/4.
 fixedV=pi/6.
 fixedphi=0.
 fixedxi=acos(cos(fixedZ)*cos(fixedV) + sin(fixedZ)*sin(fixedV)*cos(fixedphi))
 fixedkgeo=Kgeo(fixedZ,fixedZ,fixedxi)
 fixedkvol=Kvol(fixedZ,fixedZ,fixedxi)
 percentil=2

 call getarg(1,mes)
 call getarg(2,ano)
 call getarg(3,outputPrefix)

 WRITE(CTAUWL0,'(F6.3)') TAUWL0

 IF(USELMAX.EQ.1)THEN
   WRITE(CLMAX,'(I4)') LMAX
 ELSE
   CLMAX='orig'
 ENDIF

 IF(USEL0.EQ.1)THEN
   WRITE(CL0CEFF,'(I4)') L0CEFF
 ELSE
   CL0CEFF='noL0'
 ENDIF

 IF(USEL0.EQ.1)THEN
   VERSION_RESUME='Version feature/broad_albedo (min brdf) with L0 = '//CL0CEFF//', TAUWL0 = '//CTAUWL0//' and LMAX = '//CLMAX
 ELSE
   VERSION_RESUME='Version feature/broad_albedo (min brdf) without L0 and with LMAX = '//CLMAX
 ENDIF
 print*,VERSION_RESUME


 dir_netcdf = trim(outputPrefix)//"/modelo"//ano//"/"//mes//"/images/netcdf/"
 dir_maxmin = trim(outputPrefix)//"/modelo"//ano//"/"//mes//"/maxmin/"
 img_netcdf = trim(outputPrefix)//"/modelo"//ano//"/"//mes//"/images/netcdf/image_horas.dat"
 brdfdir=trim(outputPrefix)//"/brdf/CDO/"

 brdfile = 'BRDF_parameters_Band1_'//mes//'.nc'
 ncbrdf=trim(brdfdir)//trim(brdfile)
 print*,'BRDF file:',ncbrdf

 !------------	Open netcdf com dados BDRF
 call check(nf90_open(ncbrdf, nf90_nowrite, ncbrdfid) )
 call check(nf90_inquire(ncbrdfid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))
 call check(nf90_inquire(ncbrdfid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))
 allocate(LL(nvars_in))

 do i=1,nvars_in
    call check(nf90_inquire_variable(ncbrdfid,i,name = LL(i)))
    print*,LL(i)
    if (LL(i).EQ.NTIME_NAME) then
      call check(nf90_inq_varid(ncbrdfid, LL(i), ntime_varid))
      call check(nf90_inq_dimid(ncbrdfid, NTIME_NAME, ntime_dimid))
      call check(nf90_inquire_dimension(ncbrdfid,ntime_dimid,len=ntime))
    else if (LL(i).EQ.BLAT_NAME) then
      call check(nf90_inq_varid(ncbrdfid, LL(i), blat_varid))
      call check(nf90_inq_dimid(ncbrdfid, BLAT_NAME, blat_dimid))
      call check(nf90_inquire_dimension(ncbrdfid,blat_dimid,len=bnlat))
    else if (LL(i).EQ.BLON_NAME) then
      call check(nf90_inq_varid(ncbrdfid, LL(i), blon_varid))
      call check(nf90_inq_dimid(ncbrdfid, BLON_NAME, blon_dimid))
      call check(nf90_inquire_dimension(ncbrdfid,blon_dimid,len=bnlon))
    else if (LL(i).EQ.FISO_NAME) then
      call check(nf90_inq_varid(ncbrdfid, LL(i), fiso_varid))
      call check(nf90_inquire_attribute(ncbrdfid, fiso_varid, "scale_factor",len = scalelength))
    else if (LL(i).EQ.FGEO_NAME) then
      call check(nf90_inq_varid(ncbrdfid, LL(i), fgeo_varid))
      call check(nf90_inquire_attribute(ncbrdfid, fiso_varid, "scale_factor",len = scalelength))
    else if (LL(i).EQ.FVOL_NAME) then
      call check(nf90_inq_varid(ncbrdfid, LL(i), fvol_varid))
      call check(nf90_inquire_attribute(ncbrdfid, fiso_varid, "scale_factor",len = scalelength))
    endif
 enddo
   allocate(fiso_scale_factor(scalelength))
   allocate(fgeo_scale_factor(scalelength))
   allocate(fvol_scale_factor(scalelength))

   call check(nf90_get_att(ncbrdfid, fiso_varid, "scale_factor",fiso_scale_factor))
   call check(nf90_get_att(ncbrdfid, fgeo_varid, "scale_factor",fgeo_scale_factor))
   call check(nf90_get_att(ncbrdfid, fvol_varid, "scale_factor",fvol_scale_factor))

   allocate(blats(bnlat))
   allocate(blons(bnlon))


   call check(nf90_get_var(ncbrdfid,blat_varid,blats))
   call check(nf90_get_var(ncbrdfid,blon_varid,blons))

   allocate(fiso(bnlon, bnlat))
   allocate(fgeo(bnlon, bnlat))
   allocate(fvol(bnlon, bnlat))


   call check(nf90_get_var(ncbrdfid,fiso_varid,fiso))
   call check(nf90_get_var(ncbrdfid,fgeo_varid,fgeo))
   call check(nf90_get_var(ncbrdfid,fvol_varid,fvol))
   call check(nf90_close(ncbrdfid))

   sub_i=1
   sub_j=1
   do j=JMIN,JMAX
      do i=IMIN,IMAX
        xfiso(sub_i,sub_j) = fiso(i,j) * fiso_scale_factor(1)
        xfgeo(sub_i,sub_j) = fgeo(i,j) * fgeo_scale_factor(1)
        xfvol(sub_i,sub_j) = fvol(i,j) * fvol_scale_factor(1)
        fixedbrdf(sub_i,sub_j)=xfiso(sub_i,sub_j)+xfgeo(sub_i,sub_j)*fixedkgeo+xfvol(sub_i,sub_j)*fixedkvol
        sub_i = sub_i + 1
      enddo!j
      sub_j = sub_j + 1
      sub_i=1
   enddo!i

   deallocate(fiso)
   deallocate(fgeo)
   deallocate(fvol)
   deallocate(blats)
   deallocate(blons)
   deallocate(fiso_scale_factor)
   deallocate(fgeo_scale_factor)
   deallocate(fvol_scale_factor)
print*, "BRDF data processed"


!**************************************************************************
!Abre os netcdf's
open(25,file=img_netcdf)
hour_loop: do
     read(25,111,iostat=ierr)fnetcdf
     111 format(22A)
        print*,'imagens da hora em ',fnetcdf
     if((ierr/=0) .OR. (len(trim(fnetcdf)).LT.22)) exit
     cont = 0

     open(13,file=trim(dir_netcdf)//fnetcdf)
     sat_loop: do
         read(13,100,iostat=ierr2)arq_netcdf
         100 format(18A)
         cont = cont + 1
         if(ierr2/=0) exit
         hora = arq_netcdf(12:15)
         dia  = arq_netcdf(4:6)
         ano  = arq_netcdf(7:10)

         print*,'arquivo ',arq_netcdf
         name_netcdf=trim(dir_netcdf)//arq_netcdf

         call check (nf90_open(name_netcdf,nf90_nowrite,ncid))
         call check(nf90_inquire(ncid, ndims_in, nvars_in, ngatts_in, unlimdimid_in))

         call check(nf90_inq_varid(ncid, "latitude", lat_varid))
         call check(nf90_inq_varid(ncid, "longitude", lon_varid))
         call check(nf90_inq_varid(ncid,"imagem_ir", ir_varid))
         call check(nf90_inq_varid(ncid,"imagem_vi", vi_varid))
         call check(nf90_inq_varid(ncid,"angulo_sol", ang_sol_varid))
         call check(nf90_inq_varid(ncid,"angulo_satelite", ang_sat_varid))
         call check(nf90_inq_varid(ncid,"angulo_satelite_sol", ang_sat_sol_varid))

         call check(nf90_inquire_dimension(ncid,lat_varid,len=ilat))
         call check(nf90_inquire_dimension(ncid,lon_varid,len=ilon))
         allocate(lats(ilat))
         allocate(lons(ilon))
         allocate(vi(ilon, ilat))
         allocate(solarZ(ilon, ilat))
         allocate(viewZ(ilon, ilat))
         allocate(xi(ilon, ilat))

         call check(nf90_get_var(ncid,lat_varid,lats))
         call check(nf90_get_var(ncid,lon_varid,lons))
         call check(nf90_get_var(ncid,vi_varid,vi))
         call check(nf90_get_var(ncid,ang_sol_varid,solarZ))
         call check(nf90_get_var(ncid,ang_sat_varid,viewZ))
         call check(nf90_get_var(ncid,ang_sat_sol_varid,xi))

         call check( nf90_close(ncid) )

      !****************************************************************************
      solarZ=solarZ*pi/180.
      viewZ=viewZ*pi/180.
      xi=xi*pi/180.

      sub_i=1
      sub_j=1
      do j=JMIN,JMAX
         do i=IMIN,IMAX
            Xvi(sub_i,sub_j) = vi(i,j)
            XsolarZ(sub_i,sub_j) = solarZ(i,j)
            XviewZ(sub_i,sub_j) = viewZ(i,j)
            Xxi(sub_i,sub_j) = xi(i,j)
            XLON(sub_i)   = lons(i)
            sub_i = sub_i + 1
         enddo!j
         XLAT(sub_j)   = lats(j)
         sub_j = sub_j + 1
         sub_i=1
      enddo!i

      deallocate(lats)
      deallocate(lons)
      deallocate(vi)
      deallocate(solarZ)
      deallocate(viewZ)
      deallocate(xi)

      VALIDMASK=((Xvi /= -99).AND.(Xvi /= -32768).AND.(XsolarZ < pi/3.))
      WHERE (VALIDMASK) ISVALID=1
      NVALID=NVALID+ISVALID


      do j=1,JRANGE
         do i=1,IRANGE
           if (VALIDMASK(i,j)) then
             Xvinorm(i,j)=Xvi(i,j)*fixedbrdf(i,j)*cos(fixedZ)/(brdf(xfiso(i,j), xfgeo(i,j), xfvol(i,j), XsolarZ(i,j),XviewZ(i,j),Xxi(i,j))*cos(XsolarZ(i,j)))
           endif
        enddo
      enddo
      ! print*, 'file processed'
       do j=1,JRANGE
           do i=1,IRANGE
              if (VALIDMASK(i,j)) then
                !!! Only Zenith angles lower than 60 degrees are considered
                 if (Xvinorm(i,j).LT.MAXVAL(minimos(i,j,:)))then
                    maxpos= MAXLOC(minimos(i,j,:))
                    minimos(i,j,maxpos) = Xvinorm(i,j) !REPLACE MAX VALUE IN ARRAY *2 BY LOWER-VALUE Xvi
                 endif
              endif
           enddo
       enddo
       if (USELMAX.EQ.0) then
         do j=1,JRANGE
             do i=1,IRANGE
               if (Xvi(i,j) .GT. maximo(i,j))then
                  maximo(i,j) = Xvi(i,j)
               endif
             enddo
         enddo
       endif
      !****************************************************************************
    enddo sat_loop
     close(13)
     if (USELMAX.EQ.0) then
       do j=1,JRANGE
          do i=1,IRANGE
            if(maximo(i,j) == -32767)then
               maximo(i,j) = -99
            endif
          enddo
       enddo
     endif

    !****************************************************************************
     if (USELMAX.EQ.0) then
       !if (cont >= 10)then
         print*,'Saving maximum for ',mes,' ',ano,' ',hora
         file_saida = trim(dir_maxmin)//mes//ano//'.'//hora//'_max.nc'

      ! salvando arquivos
        call check(nf90_create(file_saida,nf90_clobber,ncidm))

       ! Define as dimensões.
        call check( nf90_def_dim(ncidm,LAT_NAME, NLAT, lat_dimid))
        call check( nf90_def_dim(ncidm,LON_NAME, NLON, lon_dimid))

       ! Define as variáveis de coordenadas. Manterão as informações das coordenadas.
       ! Sendo que uma "varid" é retornada para cada.
        call check( nf90_def_var(ncidm, LAT_NAME, NF90_REAL, lat_dimid, lat_varid) )
        call check( nf90_def_var(ncidm, LON_NAME, NF90_REAL, lon_dimid, lon_varid) )

       ! Alinha as unidades aos dados das coordenadas, isso para cada uma.
        call check( nf90_put_att(ncidm, lat_varid, UNITS, LAT_UNITS) )
        call check( nf90_put_att(ncidm, lon_varid, UNITS, LON_UNITS) )

       ! Define as variáveis netCDF para cobertuna de nuvens.
        dimids = (/ lon_dimid, lat_dimid /)
        call check( nf90_def_var(ncidm, MAX_NAME, NF90_INT, dimids, max_varid))


       ! Alia às variáveis as unidades correspondentes.
        call check( nf90_put_att(ncidm, max_varid, LONGNAME, MAX_LONGNAME) )
        call check( nf90_put_att(ncidm, max_varid, UNITS, MAX_UNITS) )
        call check( nf90_put_att(ncid, max_varid,   OFFSET, offset_factor))
        call check( nf90_put_att(ncid, max_varid,   SCALE, scale_factor))

        call check( nf90_put_att(ncid, NF90_GLOBAL, VERSION_NAME, VERSION_RESUME) )
       ! Fim do modo de definição.
        call check( nf90_enddef(ncidm) )

       ! Escreve os dados das variáveis de coordenadas.
       ! Coloca as latitudes e longituides no grid do arquivo netCDF.
        call check( nf90_put_var(ncidm, lat_varid, XLAT) )
        call check( nf90_put_var(ncidm, lon_varid, XLON) )

        ! Escreve os outros dados pretendidos.
        ! Isto escreverá os dados de cobertuna de nuvens.
        ! A lista de dados são do mesmo tamanho que as variáveis netCDF foram definidas.

        call check( nf90_put_var(ncidm, max_varid, maximo) )

        ! Fecha o arquivo.
        ! Isso faz com que o netCDF limpe toda a memória temporária usada até agora e
        ! dá a certeza de que os dados são realmente escritos no disco.
        call check( nf90_close(ncidm) )

       !endif

          maximo=-32767
      endif
enddo hour_loop
close(25)

!****************************************************************************
 ! PMIN_POS(:,:)=NVALID(:,:)*2/100 + 1
! print*,NVALID(500,500),NVALID(1000,1000)
   do j=1,JRANGE
      do i=1,IRANGE
        PMIN_POS(i,j)=NVALID(i,j)*percentil/100 + 1
        minimos(i,j,:)=sortedarray(minimos(i,j,:))
        if (PMIN_POS(i,j).GT.0) then
          minP2(i,j)=minimos(i,j,PMIN_POS(i,j))
        else
          print*,'sem dados validos para ',i, j, NVALID(i,j)
          STOP
        endif
      enddo
   enddo

   print*,'Saving P2 BRDF'
   file_saida = trim(dir_maxmin)//mes//ano//'_brdf_p2.nc'
   print*,file_saida
  ! salvando arquivos
   call check(nf90_create(file_saida,nf90_clobber,ncidm))

  ! Define as dimensões.
   call check( nf90_def_dim(ncidm,LAT_NAME, NLAT, lat_dimid))
   call check( nf90_def_dim(ncidm,LON_NAME, NLON, lon_dimid))

  ! Define as variáveis de coordenadas. Manterão as informações das coordenadas.
  ! Sendo que uma "varid" é retornada para cada.
   call check( nf90_def_var(ncidm, LAT_NAME, NF90_REAL, lat_dimid, lat_varid) )
   call check( nf90_def_var(ncidm, LON_NAME, NF90_REAL, lon_dimid, lon_varid) )

  ! Alinha as unidades aos dados das coordenadas, isso para cada uma.
   call check( nf90_put_att(ncidm, lat_varid, UNITS, LAT_UNITS) )
   call check( nf90_put_att(ncidm, lon_varid, UNITS, LON_UNITS) )

  ! Define as variáveis netCDF para cobertuna de nuvens.
   dimids = (/ lon_dimid, lat_dimid /)
   call check( nf90_def_var(ncidm, P2_NAME, NF90_REAL, dimids, p2_varid))


  ! Alia às variáveis as unidades correspondentes.
   call check( nf90_put_att(ncidm, p2_varid, LONGNAME, p2_LONGNAME) )
   call check( nf90_put_att(ncidm, p2_varid, UNITS, p2_UNITS) )
   call check( nf90_put_att(ncid, p2_varid,   OFFSET, offset_factor))
   call check( nf90_put_att(ncid, p2_varid,   SCALE, scale_factor))

    call check( nf90_put_att(ncid, NF90_GLOBAL, VERSION_NAME, VERSION_RESUME) )
  ! Fim do modo de definição.
   call check( nf90_enddef(ncidm) )

  ! Escreve os dados das variáveis de coordenadas.
  ! Coloca as latitudes e longituides no grid do arquivo netCDF.
   call check( nf90_put_var(ncidm, lat_varid, XLAT) )
   call check( nf90_put_var(ncidm, lon_varid, XLON) )

   ! Escreve os outros dados pretendidos.
   ! Isto escreverá os dados de cobertuna de nuvens.
   ! A lista de dados são do mesmo tamanho que as variáveis netCDF foram definidas.
   call check( nf90_put_var(ncidm, p2_varid, minP2) )

   ! Fecha o arquivo.
   ! Isso faz com que o netCDF limpe toda a memória temporária usada até agora e
   ! dá a certeza de que os dados são realmente escritos no disco.
   call check( nf90_close(ncidm) )


!****************************************************************************
contains

  FUNCTION sortedarray(array)
    IMPLICIT NONE
    INTEGER::J,K
    REAL(KIND=8) ::temp,array(:)
    REAL(KIND=8) ::sortedarray(size(array))

    L1: DO J = 1,UBOUND(array,1)-1
      L2: DO K = J+1,UBOUND(array,1)
        IF(array(J) > array(K)) THEN
          temp = array(K)
          array(K) = array(J)
          array(J) = temp
        ENDIF
      END DO L2
    END DO L1
    sortedarray=array
  END FUNCTION sortedarray

  !**************************************************************************
    real function brdf(fiso, fgeo, fvol, solarZ,viewZ,xi)
      real*8, intent (in) :: fiso, fgeo, fvol, solarZ,viewZ,xi

      brdf=fiso+fgeo*Kgeo(solarZ,viewZ,xi)+fvol*Kvol(solarZ,viewZ,xi)

    end function brdf
   !*************************************************

    real function Kvol(solarZ,viewZ,xi)
      REAL(KIND=8), intent (in) :: solarZ,viewZ,xi
      REAL(KIND=8), parameter ::pi =  3.1415927

      ! print*,solarZ*180./pi,viewZ*180./pi, phi*180./pi, xi*180./pi, xi, cos(xi),cos(solarZ)*cos(viewZ) + sin(solarZ)*sin(viewZ)*cos(phi), sin(xi)
      Kvol=((pi/2.-xi)*cos(xi)+sin(xi))/(cos(solarZ)+cos(viewZ))-pi/4.
      ! print*,(pi/2.-xi),cos(xi),sin(xi),(cos(solarZ)+cos(viewZ)),Kvol
      ! pause
    end function Kvol
   !--------------------------------------------
    real function Kgeo(solarZ,viewZ,xi)
      REAL(KIND=8), intent (in) :: solarZ,viewZ,xi
      REAL(KIND=8), parameter ::pi = 3.1415927
      REAL(KIND=8), parameter :: hb=2.000000  !h/b
      REAL(KIND=8), parameter :: br=1.000000   !b/r
      REAL(KIND=8)::O,D,t,cost,phi

      phi=acos((cos(xi)-cos(solarZ)*cos(viewZ))/(sin(solarZ)*sin(viewZ)))
      D=sqrt((tan(solarZ))**2+(tan(viewZ))**2-2*tan(solarZ)*tan(viewZ)*cos(phi))
      cost=hb*sqrt(D**2+(tan(solarZ)*tan(ViewZ)*sin(phi))**2)/((1/cos(solarZ))+(1/cos(viewZ)))
      t=acos(min(max(cost,-1.),1.))
      ! if(cost.LE.1.0000)then
      !   t=acos(cost)
      ! else
      !   t=asin(1-cost)
      ! endif
      O=(1./pi)*(t-sin(t)*cos(t))*((1/cos(solarZ))+(1/cos(viewZ)))
      Kgeo=O-(1./cos(solarZ))-(1./cos(viewZ))+(1./2.)*(1+cos(xi))*(1/cos(viewZ))*(1/cos(solarZ))

    end function Kgeo
!--------------------------------------------
subroutine check(status)
   integer, intent (in) :: status
   if(status /= nf90_noerr) then
     print *, trim(nf90_strerror(status))
     stop 2
   end if
 end subroutine check
!*************************************************
end program
